/*
 * @(#)Anthill.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import org.apache.log4j.Category;
import com.urbancode.lib.butler.Butler;
import com.urbancode.lib.butler.ButlerException;
import com.urbancode.lib.registry.Registry;
import com.urbancode.lib.registry.RegistryEntry;
import com.urbancode.lib.version.VersionReader;
import com.urbancode.anthill.util.Queue;
import com.urbancode.anthill.util.FileUtils;
import java.io.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Date;
import java.util.Vector;
/**
 * This class scans the project directory for all .anthill files and for each 
 * one instantiates an AnthillProject.  It then determines whether each  
 * project needs to be built and if so, builds the project.
 *
 * @version $Revision: 1.11 $
 * @author 
 */
public class Anthill {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(Anthill.class.getName());
    
    // the system property application name - used with Butler
    static public final String APP_NAME = "anthill";
    
    static public final String BUILD_INTERVAL_KEY  = "build.interval";
    
    static public final String ANTHILL_ROOT_DIR_KEY = "anthill.root.dir";

    static public final String NEW_PROJECT_KEY      = "_temp";

    static public final String NEW_SCHEDULE_KEY     = "_temp";

    static public final String NEW_DEPENDENCY_GROUP_KEY = "_temp";

    static public Anthill anthill = null;
    
    // print out the version of Anthill
    static private String versionStr = null;
    static {
        System.out.println("Anthill version: "+getVersion() + " loaded.");
    }
    
    //--------------------------------------------------------------------------
    /**
     * Returns the version of this software.
     *
     * @return a String representing the version of this software
     */
    static public String getVersion() {
        if (versionStr == null) {
            ClassLoader loader = AnthillProject.class.getClassLoader();
            InputStream in = loader.getResourceAsStream("anthill.version");
            versionStr = VersionReader.readVersionStr(in);
        }
        return versionStr;
    }

    //--------------------------------------------------------------------------
    public static Anthill getAnthill() 
    throws Exception {
        if (anthill == null) {
            anthill = new Anthill();
        }
        return anthill;
    }
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected File anthillRootDir = null;
    protected File configDir = null;
    protected File projectsDir = null;
    protected File publishDir = null;
    protected AnthillConfig defaults = null;
    protected Map name2project = new TreeMap();
    protected Map name2schedule = new TreeMap();
    protected Map name2group = new TreeMap();
    protected Queue queue = new Queue();
    protected BuildDaemon buildDeamon = null;
    
    //--------------------------------------------------------------------------
    private Anthill() 
    throws Exception {
        String anthillRootDirName = System.getProperty(ANTHILL_ROOT_DIR_KEY);
        anthillRootDir = new File(anthillRootDirName);
        if (!anthillRootDir.exists() || !anthillRootDir.isDirectory()) {
            throw new IllegalStateException("The Anthill root directory is " +
                                            "not valid.");
        }
        System.out.println("Anthill root dir: " +
                            anthillRootDir.getAbsolutePath());

        // set the conf.dir.Anthill System property for Butler
        configDir = new File(anthillRootDir, "conf");
        String configDirStr = configDir.getAbsolutePath();
        System.setProperty(Butler.BUTLER_ROOT_PREFIX + APP_NAME, configDirStr);
        
        initialize();
    }

    //--------------------------------------------------------------------------
    public AnthillProject createProject(String name) 
    throws Exception {
        if (name2project.containsKey(name)) {
            throw new IllegalArgumentException("An AnthillProject with the " +
                            "name " + name + " already exists.");
        }
        AnthillProject project = new AnthillProject(this, name);
        name2project.put(name, project);
        return project;
    }
    
    //--------------------------------------------------------------------------
    public AnthillSchedule createSchedule(String name) 
    throws Exception {
        if (name2schedule.containsKey(name)) {
            throw new IllegalArgumentException("An AnthillSchedule with the " +
                            "name " + name + " already exists.");
        }
        AnthillSchedule schedule = new AnthillSchedule(this, name);
        name2schedule.put(name, schedule);
        return schedule;
    }
    
    //--------------------------------------------------------------------------
    public DependencyGroup createDependencyGroup(String name)
    throws Exception {
        if (name2group.containsKey(name)) {
            throw new IllegalArgumentException("A DependencyGroup with the " +
                "name " + name + " already exists.");
        }
        File file = new File(name + DependencyGroup.DEPENDENCY_GROUP_FILE_SUFFIX);
        DependencyGroup group = new DependencyGroup(this, file);
        name2group.put(name, group);
        log.debug("added group to map *" + name + "*");
        return group;
    }
    

    public void deleteProject(String name)
    throws Exception {
        if (!name2project.containsKey(name)) {
            throw new IllegalArgumentException("Project: " + name + " doesn't exist.");
        }
        //have to delete project from its schedule        
        AnthillProject proj = getProject(name);
        AnthillSchedule schedule = proj.getSchedule();
        schedule.remove(proj);
        //  here you can do the delete
        File tempProjectFile = new File(getAnthillProjectsDir() + File.separator + name + ".anthill");
        if (!tempProjectFile.exists()) {
            throw new IllegalArgumentException("Project file: " + tempProjectFile.getAbsolutePath() +
                                               " for project: " + name + " doesn't exist.");
        }
        // else the file exists, delete it
        if (tempProjectFile.delete()) {
            // then here you also want to remove it from the map...I think
            name2project.remove(name);
        }
        else {
            // ?
            log.error("Unable to delete project file: " + 
                      tempProjectFile.getAbsolutePath());
        }
        
        
    }
    
    public void deleteSchedule(String name)
    throws Exception {
        
        Iterator itr = getProjectIterator();
        while (itr.hasNext()) {
             AnthillProject project = (AnthillProject)itr.next();
             AnthillSchedule schedule = project.getSchedule();
             String projScheduleName = schedule.getScheduleName();
             if (name.equals(projScheduleName)) {
                 throw new IllegalArgumentException("Schedule - " + name + " - is in use by project " + 
                 project.getProjectName());
             }
        }
        if (!name2schedule.containsKey(name)) {
            throw new IllegalArgumentException("Schedule: " + name + " doesn't exist.");
        }
        // else, here you can delete the schedule
        File tempScheduleFile = new File(getAnthillConfigDir()+ File.separator + name + ".schedule");
        if (!tempScheduleFile.exists()) {
            throw new IllegalArgumentException("Schedule file: " + tempScheduleFile.getAbsolutePath() + 
                                               " for schedule: " + name + " doesn't exist.");
        }
        
        if (tempScheduleFile.delete()) {
            log.debug("deleted schedule " + name);             
            name2schedule.remove(name);
        }
        else {
            // what to do here? 
            log.error("Unable to delete schedule file: " + 
                      tempScheduleFile.getAbsolutePath());
        }
    }
    
    public void deleteDependencyGroup(String name)
    throws Exception {
        if (!name2group.containsKey(name)) {
            throw new IllegalArgumentException("Dependency group: " + name + " doesn't exist.");
        }
        //remove from its build schedule
        DependencyGroup depGroup = getDependencyGroup(name);
        AnthillSchedule schedule = depGroup.getSchedule();
        schedule.remove(depGroup);
        
        //delete the group
        File tempDependencyGroupFile = new File(getAnthillProjectsDir()+ File.separator + name + ".dependencies");
        if (!tempDependencyGroupFile.exists()) {
            throw new IllegalArgumentException("Dependency Group file: " + 
                                              tempDependencyGroupFile.getAbsolutePath() +
                                              " for dependency group: " + name + " doesn't exist.");
        }
        
        if (tempDependencyGroupFile.delete()) {
            name2group.remove(name);
        }
        else {
            // what here? 
            log.error("Unable to delete dependency group file: " +
                      tempDependencyGroupFile.getAbsolutePath());
        }
    }
    

    //--------------------------------------------------------------------------
    public AnthillConfig getAnthillConfig() {
        return defaults;
    }

    //--------------------------------------------------------------------------
    public File getAnthillConfigDir() {
        return configDir;
    }
    
    //--------------------------------------------------------------------------
    public File getAnthillDefaultPublishDir() {
        return publishDir;
    }
    
    //--------------------------------------------------------------------------
    public File getAnthillProjectsDir() {
        return projectsDir;
    }
    
    //--------------------------------------------------------------------------
    public File getAnthillRootDir() {
        return anthillRootDir;
    }
    
    //--------------------------------------------------------------------------
    public AnthillProject getProject(String name) {
        if (!name2project.containsKey(name)) {
            try {
                refreshProjectMap();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return (AnthillProject)name2project.get(name);
    }
    
    //--------------------------------------------------------------------------
    public Iterator getProjectIterator() {
        try {
            refreshProjectMap();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return name2project.values().iterator();
    }
    
    //--------------------------------------------------------------------------
    public DependencyGroup getDependencyGroup(String name) {
        if (!name2group.containsKey(name)) {
            try {
                refreshDependencyGroupMap();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return (DependencyGroup)name2group.get(name);
    }
    
    //--------------------------------------------------------------------------
    public Iterator getDependencyGroupIterator() {
        try {
            refreshDependencyGroupMap();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return name2group.values().iterator();
    }
    
    //--------------------------------------------------------------------------
    public Queue getBuildQueue() {
        return queue;
    }
    
    //--------------------------------------------------------------------------
    public AnthillSchedule getSchedule(String name) {
        if (!name2schedule.containsKey(name)) {
            try {
                refreshScheduleMap();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
        return (AnthillSchedule)name2schedule.get(name);
    }
    
    //--------------------------------------------------------------------------
    public Iterator getScheduleIterator() {
        try {
            refreshScheduleMap();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return name2schedule.values().iterator();
    }
        
    //--------------------------------------------------------------------------
    public void runDaemon() 
    throws Exception {
        log.debug("runDaemon");
        
        // start the build daemon
        buildDeamon = new BuildDaemon(queue);
        buildDeamon.start();
//moved start to refreshScheduleMap - Matt        
        // start each of the schedules
/*        Iterator itr = this.getScheduleIterator();
        AnthillSchedule schedule = null;
            while (itr.hasNext()) {
            schedule = (AnthillSchedule)itr.next();
            schedule.start();
        }*/        
    }

    //--------------------------------------------------------------------------
    public Buildable getProjectBeingBuilt() { 
        return buildDeamon.getProjectBeingBuilt();
    }
    
    //**************************************************************************
    // PROTECTED
    //**************************************************************************

    //--------------------------------------------------------------------------
    protected void doProjectNameChange(String oldName) {
        AnthillProject project = (AnthillProject)name2project.get(oldName);
        if (project == null) {
            throw new IllegalArgumentException("Project with name " + oldName + 
                                               " does not exist.");
        }
        name2project.remove(oldName);
        name2project.put(project.getProjectName(), project);
    }
    
    //--------------------------------------------------------------------------
    protected void doScheduleNameChange(String oldName) {
        AnthillSchedule schedule = (AnthillSchedule)name2schedule.get(oldName);
        if (schedule == null) {
            throw new IllegalArgumentException("Schedule with name " + oldName + 
                                               " does not exist.");
        }
        name2schedule.remove(oldName);
        name2schedule.put(schedule.getScheduleName(), schedule);
    }
    
    //--------------------------------------------------------------------------
    protected void doDependencyGroupNameChange(String oldName) {
        DependencyGroup group = (DependencyGroup) name2group.get(oldName);
        if (group == null) {
            throw new IllegalArgumentException("DependencyGroup with name " + oldName + 
                                               " does not exist.");
        }
        name2group.remove(oldName);
        name2group.put(group.getName(), group);
    } 
    
    //--------------------------------------------------------------------------
    protected void initialize() 
    throws Exception {
        // load Anthill's Butler
        // initialize Butler for log4j logging
        Butler butler = null;
        try {
            butler = Butler.getButler(APP_NAME);
        } catch (ButlerException e) {
            log.error("Butler Initialization Error: " + e.getMessage(), e);
            System.exit(1);
        }
        
        // load the default properties for Anthill
        Registry defaultRegistry = butler.getRegistry();
        defaults = new AnthillConfig(this, defaultRegistry);

        // get the directories
        this.projectsDir = defaults.getProjectsDir();
        this.publishDir = defaults.getDefaultPublishDir();
        
        // get the schedules
        refreshScheduleMap();
        
        // get the projects
        refreshProjectMap();
        
        // get the groups
        refreshDependencyGroupMap();
    }
    
    //--------------------------------------------------------------------------
    protected void refreshProjectMap() 
    throws Exception {
        File[] fileList = projectsDir.listFiles();
        AnthillProject project = null;
        String projectName = null;
        int index = 0;
        for (int i = 0; i < fileList.length; i++) {
            // make sure the file name ends with the appropriate suffix
            if (fileList[i].getName().endsWith
                                         (ProjectProperties.ANTHILL_FILE_SUFFIX)) {
                // create the project name from the file name
                // strip off the extension
                projectName = FileUtils.getNameMinusExtension(fileList[i]);

                // check whether we already have the project, 
                // if not then create an AnthillProject for that file
                if (!name2project.containsKey(projectName)) {
                    try {
                        project = new AnthillProject(this, fileList[i]);
                        name2project.put(projectName, project);
                    } catch (Exception e) {
                        log.error("Could not create Anthill project.", e);
                        throw e;
                    }
                }
            }
        }
    }

    //--------------------------------------------------------------------------
    protected void refreshScheduleMap() 
    throws Exception {
        // load the schedules
        File[] fileList = configDir.listFiles();
        AnthillSchedule schedule = null;
        String scheduleName = null;
        int index = 0;
        for (int i = 0; i < fileList.length; i++) {
            // make sure the file name ends with the appropriate suffix
            if (fileList[i].getName().endsWith
                                       (AnthillSchedule.SCHEDULE_FILE_SUFFIX)) {
                // create the schedule name from the file name
                // strip off the extension
                scheduleName = FileUtils.getNameMinusExtension(fileList[i]);
                
                // check whether we already have the project, 
                // if not then create an AnthillProject for that file
                if (!name2schedule.containsKey(scheduleName)) {
                    try {
                        schedule = new AnthillSchedule(this, scheduleName, 
                                                       fileList[i]);
                        schedule.start();
                        name2schedule.put(scheduleName, schedule);
                    } catch (Exception e) {
                        log.error("Could not create Anthill schedule.", e);
                        throw e;
                    }
                }
            }
        }
    }
    
    //--------------------------------------------------------------------------
    protected void refreshDependencyGroupMap() 
    throws Exception {
        File[] fileList = projectsDir.listFiles();
        DependencyGroup group = null;
        String groupName = null;
        int index = 0;
        for (int i = 0; i < fileList.length; i++) {
            // make sure the file name ends with the appropriate suffix
            if (fileList[i].getName().endsWith
                                         (DependencyGroup.DEPENDENCY_GROUP_FILE_SUFFIX)) {
                // create the group name from the file name
                // strip off the extension
                groupName = FileUtils.getNameMinusExtension(fileList[i]);

                // check whether we already have the group, 
                // if not then create an DependencyGroup for that file
                if (!name2group.containsKey(groupName)) {
                    try {
                        group = new DependencyGroup(this, fileList[i]);
                        name2group.put(groupName, group);
                        log.debug("added group to map *" + groupName + "*");
                    } catch (Exception e) {
                        log.error("Could not create DependencyGroup.", e);
                        throw e;
                    }
                }
            }
        }
    }

}