/*
 * @(#)AnthillConfig.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import org.apache.log4j.Category;
import com.urbancode.lib.registry.Registry;
import com.urbancode.lib.registry.RegistryEntry;
import com.urbancode.anthill.util.FileUtils;
import java.io.*;
/**
 * @version $Revision: 1.10 $
 * @author 
 */
public class AnthillConfig {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillConfig.class.getName());
    
    // the registry entry key that serves as the base for all the properties
    static public final String REGISTRY_ENTRY_KEY = "anthill";

    static public final String JAVA_HOME        = "jdk.home";
    //static public final String LOG_PATH         = "log.path";
    static public final String ANT_HOME         = "ant.home";

    static public final String PROJECTS_DIR     = "projects.dir";

    static public final String PUBLISH_DIR      = "publish.dir.default";  
    
    // the PATH_SEPARATOR system property
    static public final String PATH_SEPARATOR   = "path.separator";

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    private Anthill anthill = null;
    private String jdkHome = null;
    private String defaultUser = null;
    //private String logDirName = null;
    private String antClassPath = null;
    private RegistryEntry regEntry = null;
    private RegistryEntry rootEntry = null;
    private Registry registry = null;
    private File projectsDir = null;
    private File publishDir = null;
                   
    //--------------------------------------------------------------------------
    public AnthillConfig(Anthill anthill, Registry registry) {
        this.anthill = anthill;
        this.registry = registry;
        this.regEntry = registry.getRegistryEntry(REGISTRY_ENTRY_KEY);
        this.rootEntry = registry.getRootRegistryEntry();
        calculateJdkHome();
        //claculateLogDirName();
        calculateAntClassPath();
        calculateProjectsDir();
        calculateDefaultPublishDir();
       
    }

    //--------------------------------------------------------------------------
    public File getProjectsDir() {
        return projectsDir;
    }
    
    //--------------------------------------------------------------------------
    public File getDefaultPublishDir() {
        return publishDir;
    }
    
    //--------------------------------------------------------------------------
    /**
     *
     */
    public String getProperty(String name) {
        String value = null;
        
        // first try getting the value from the default properties
        value = regEntry.getKeyValue(name);
        
        // if there is not such project property, try the system properties
        if (value == null) {
            value = System.getProperty("anthill." + name);
        }
        
        return value;
    }

    //--------------------------------------------------------------------------
    public void setProperty(String name, String value) {
        regEntry.setKeyValue(name, value);
        log.debug("setProperty name: "+name+", value: "+value);
    }
    
    //--------------------------------------------------------------------------
    public void save() 
    throws Exception {
        registry.updateRegistryFile();
        calculateJdkHome();
        calculateAntClassPath();
        calculateProjectsDir();
        calculateDefaultPublishDir();
        anthill.initialize();
        log.debug("Saved");
    }
    
    //--------------------------------------------------------------------------
    protected void calculateDefaultPublishDir() {
        String publishDirName = getProperty(PUBLISH_DIR);
        if (publishDirName == null) {
            throw new IllegalArgumentException("Publish directory parameter " +
                            "must not be null.");
        }
        File tempPublishDir = FileUtils.makeProperPath
                                 (anthill.getAnthillRootDir(), publishDirName);
        if (!tempPublishDir.exists()) {
            throw new IllegalArgumentException("Publish directory parameter " +
                            "must point to an existing directory.");
        }
        if (!tempPublishDir.isDirectory()) {
            throw new IllegalArgumentException("Publish directory parameter " +
                            "must point to a directory.");
        }
        this.publishDir = tempPublishDir;
    }
    
    //--------------------------------------------------------------------------
    protected void calculateProjectsDir() {
        String projectsDirName = getProperty(PROJECTS_DIR);
        if (projectsDirName == null) {
            throw new IllegalArgumentException("Projects directory parameter " +
                            "must not be null.");
        }
        File tempProjectDir = FileUtils.makeProperPath
                                 (anthill.getAnthillRootDir(), projectsDirName);
        if (!tempProjectDir.exists()) {
            throw new IllegalArgumentException("Projects directory parameter " +
                            "must point to an existing directory.");
        }
        if (!tempProjectDir.isDirectory()) {
            throw new IllegalArgumentException("Projects directory parameter " +
                            "must point to a directory.");
        }
        this.projectsDir = tempProjectDir;
    }
    
    //--------------------------------------------------------------------------
    /**
     *
     */
    protected void calculateJdkHome() {
        // get the jdkHome
        jdkHome = getProperty(JAVA_HOME);
        if (jdkHome == null) {
            // try getting it from the system property
            jdkHome = System.getProperty("java.home");

            // check if the jdkHome is for the JRE
            // if so, then look for the JDK 
            if (jdkHome.endsWith("jre") || jdkHome.endsWith("JRE")) {
                log.debug("JdkHome points to a JRE - "+jdkHome);
                String tempJdk = jdkHome.substring(0, jdkHome.length()-3);
                log.debug("Looking for jdkHome in the parentDirectory: "+tempJdk);
                File jdkBin = FileUtils.makeProperPath
                 (anthill.getAnthillRootDir(), tempJdk + File.separator + "bin");
                if (jdkBin.exists()) {
                    // found the jdk
                    File jdkHomeFile = FileUtils.makeProperPath
                                         (anthill.getAnthillRootDir(), tempJdk);
                    jdkHome = jdkHomeFile.getAbsolutePath();
                }
            }
        }
        log.debug("jdkHome: "+jdkHome);
    }
    
    /*
     *
     *
    protected void claculateLogDirName() {
        // get the log path
        logDirName = getProperty(LOG_PATH);
        if (logDirName == null) {
            // set a default log path
            logDirName = "../logs/";
        }
        log.debug("logDirName: "+logDirName);

        // make sure the log directory exists
        try {
            File logDir = new File(logDirName);
            if (!logDir.exists()) {
                logDir.mkdir();
            }
        } catch (Exception e) {
            log.error("Error validating log directory: ", e);
        }
    } 
     */  
        
    //--------------------------------------------------------------------------
    protected void calculateAntClassPath() {
        // get the file seperator and necessary system properties
        String pathSeparator = System.getProperty(PATH_SEPARATOR);
        String antHome = getProperty(ANT_HOME);
        StringBuffer classpath = new StringBuffer();
        
        // validate the system properties ANT_HOME and JAVA_HOME
        if (antHome == null) {
            throw new IllegalStateException(ANT_HOME + " property not set.");
        }

        // make sure that antHome does not end with / or \
        if (antHome.endsWith("\\")) {
            antHome = antHome.substring(0, antHome.length() - 1);
        }
        if (antHome.endsWith("/")) {
            antHome = antHome.substring(0, antHome.length() - 1);
        }
        log.debug("AntHome: "+antHome);
        
        // we can use either the bundled Ant or a separate installation of Ant
        // Here we test to determine whether the ANT_HOME points to a 
        // separate installation or the bundled Ant.  We actually only test to 
        // determine whether it is bundled. 
        File antHomeFile = FileUtils.makeProperPath
                                         (anthill.getAnthillRootDir(), antHome);
        if (antHomeFile.exists() && antHomeFile.isDirectory()) {
            File antJarFile = new File(antHomeFile, "ant.jar");
            if (!antJarFile.exists()) {
                log.error("Can not find ant.jar in the AntHome location.");
                throw new IllegalStateException("Can not find ant.jar in the " +
                                                "AntHome location.");
            }
            // make ant home absolute
            antHome = antHomeFile.getAbsolutePath();
            
            // build the classpath necessary for Ant
            classpath.append(jdkHome).append(File.separator).append("lib")
                     .append(File.separator).append("tools.jar").append(pathSeparator);

            // scan for all jar and zip files in the directory
            String[] fileList = antHomeFile.list();
            String file = null;
            for (int i = 0; i < fileList.length; i++) {
                file = fileList[i];
                // only include jar or zip files in the classpath
                if (file.endsWith(".jar") || file.endsWith(".zip")) {
                    classpath.append(antHome).append(File.separator)
                             .append(file).append(pathSeparator);
                }
            }
        } else {
            throw new IllegalStateException("The specified AntHome does not " +
                                            "exist or is not a directory.");
        }
        
        //classpath.append("\"");
        antClassPath = classpath.toString();
        log.debug("Ant ClassPath: "+antClassPath);
    }
    
    /*
    protected String getAntLogDirName() {
        return logDirName;
    }
     */
    
    //--------------------------------------------------------------------------
    protected String getJdkHome() {
        return jdkHome;
    }

    //--------------------------------------------------------------------------
    protected String getAntClassPath() {
        return antClassPath;
    }


}
