/*
 * @(#)AnthillProject.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import org.apache.log4j.Category;
import com.urbancode.anthill.adapter.*;
import com.urbancode.anthill.util.*;
import com.urbancode.lib.registry.Registry;
import com.urbancode.lib.registry.RegistryEntry;
import java.io.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Date;
import java.util.Properties;
import java.util.Map;
import java.util.HashMap;
import java.text.SimpleDateFormat;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.AddressException;
import javax.mail.internet.MimeMessage;
import javax.mail.Session;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Transport;
/**
 * @author Ryan Smith
 * @author Maciej Zawadzki
 * @author Jason Buck
 */
public class AnthillProject implements Buildable {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillProject.class.getName());
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected Anthill anthill = null;
    protected AnthillConfig anthillDefaults = null;
    protected ProjectProperties properties = null;
    protected String projectName = null;
//    protected RepositoryAdapter repositoryAdapter = null;
//    protected VersionAdapter versionAdapter = null;
//    protected AnthillSchedule schedule = null;
    protected String latestVersion = null;
    protected String branchName = null;

    //--------------------------------------------------------------------------
    protected AnthillProject(Anthill anthill, String projectName) 
    throws Exception {
        this.anthill = anthill;
        this.anthillDefaults = anthill.getAnthillConfig();
        this.projectName = projectName;
        this.properties = new ProjectProperties(anthill, this);
    }
    
    //--------------------------------------------------------------------------
    protected AnthillProject(Anthill anthill, File regFile) 
    throws Exception {
        this.anthill = anthill;
        this.anthillDefaults = anthill.getAnthillConfig();
        this.projectName = FileUtils.getNameMinusExtension(regFile);
        this.properties = new ProjectProperties(anthill, this, regFile);
    }
        
    //--------------------------------------------------------------------------
    public String getName() {
        return getProjectName();
    }
    
    // these are for the filters, you cannot access the branch
    // name through the repository adapter before you have a properties
    // object, so this is the hack
    public String getBranchName() {
        return branchName;
    }
    // specifically this set method, which will be called from within 
    // properties (but before the constructor returns)
    public void setBranchName(String name) {
        branchName = name;
    }
    
    //--------------------------------------------------------------------------
    /** 
     * Get the arbitrary name that was associated with the AntHill project
     * @return  the arbitrary name of this AntHill project
     */
    public String getProjectName() {
        return projectName;
    }
    
    //--------------------------------------------------------------------------
    public void setProjectName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Project name cannot be null.");
        }
        String oldProjectName = projectName;
        projectName = name;
        // if the name changed, then update Anthill
        if (!projectName.equals(oldProjectName)) {
            anthill.doProjectNameChange(oldProjectName);
        }
    }
    
    //--------------------------------------------------------------------------
    public void setSchedule(AnthillSchedule schedule) {
        properties.setScheduleName(schedule.getScheduleName());
    }
    
    //--------------------------------------------------------------------------
    public AnthillSchedule getSchedule() {
        return properties.getSchedule();
    }

    //--------------------------------------------------------------------------
    public ProjectProperties getProperties() {
        return properties;
    }
        
    //--------------------------------------------------------------------------
    public String getCurrentVersion() {
        String curVer = null;
        //try {
        //    curVer = getVersionAdapter().getCurrentVersion();
        //} catch (Exception e) {
        //    log.error(e.getMessage(), e);
        //}
        curVer = getProperties().getProperty(ProjectProperties.PROJECT_VERSION);
        if (curVer == null) curVer = "(not available)";
        return curVer;
    }

    //--------------------------------------------------------------------------
    /**
     * If the project already exists, then we need to make sure that it was 
     * not renamed.  If it was renamed, then we need to delete it's old 
     * project file and save it to a new file (a file with the correct name).
     * If the project is newly created, then we need to save it to a new file.
     */
    public void save() 
    throws Exception {
        properties.save();
    }
        
    //--------------------------------------------------------------------------
    public RepositoryAdapter getRepositoryAdapter() {
        return properties.getRepositoryAdapter();
    }
    
    //--------------------------------------------------------------------------
    public VersionAdapter getVersionAdapter() {
        return properties.getVersionAdapter();
    }
    
    //--------------------------------------------------------------------------
    public Iterator getProjectTargetIterator() {
        return null;
    }

    //--------------------------------------------------------------------------
    /**
     * Implementation of Buildable
     */
    public void run() {
        releaseProject();
    }
        
    //--------------------------------------------------------------------------
    /**
     * Releases a project as next version increment
     */
    public synchronized void releaseProject() {
        releaseProject(null, null, false);
    }
    
    //--------------------------------------------------------------------------
    /**
     * Releases a project as the given version
     */
    public synchronized void releaseProject(String version) {
        releaseProject(null, version, false);
    }

    //--------------------------------------------------------------------------
    /**
     * Releases a project using the specified target (if not null) as the 
     * given version (or next increment if null) and forces the release if 
     * forceBuild is true
     */
    public synchronized void releaseProject(String target, String version, boolean forceBuild) {
        BuildDefinition buildDef = new BuildDefinition(this);
        if (target != null) {
            buildDef.setTarget(new ProjectTarget(this, target));
        }
        buildDef.setVersion(version);
        buildDef.setForceBuildFlag(forceBuild);
        
        releaseProject(buildDef);
    }
    
    //--------------------------------------------------------------------------
    /**
     * Releases a project using the directives specified by the BuildDefinition
     */
    public synchronized void releaseProject(BuildDefinition buildDef) {
        buildDef.appendLogMessage("Anthill version ")
            .appendLogMessage(Anthill.getVersion())
            .appendLogMessage("\n\n");        
        ProjectProperties properties = this.properties;

        // if we are building a specific target, allow target properties to
        // override project properties
        if (buildDef.getTarget() != null) {
            properties = properties.mergeProperties(          
                buildDef.getTarget().getRegistryEntry());

        }
        
//        releaseState.setProjectProperties(properties);

//        String versionFilePath = properties.getLocalVersionFilePath();
        RepositoryAdapter repositoryAdapter = properties.getRepositoryAdapter();
        VersionAdapter versionAdapter = properties.getVersionAdapter();
        
        log.info("Project: " + getProjectName() + " is being released.");

        checkout(buildDef);//releaseState);
        
        if (!buildDef.getErrorFlag()) {
            if (buildDef.getForceBuildFlag() || checkForBuild(buildDef)) {
                if (!buildDef.getErrorFlag()) {
                    try {
/* MBZ - 060702 - Moved to UrbancodeVersionAdapter
                        // lock the version file in the repository
                        // check lock version property first
                        if (properties.getLockVersionFlag()) {
                            log.info("Lock version file: ");
                            repositoryAdapter.lockFile(versionFilePath);
//                            releaseState.setVersionFileLockedFlag(true);
                        } else {
                            log.info("version file locking turned off");
                        }
*/
                        if (buildDef.getVersion() == null) {
                            // increment the version in the version file
                            log.info("Increment version: ");
                            String tempVersion = versionAdapter.incrementVersion();
                            buildDef.setVersion(tempVersion);
                            // buildDef.setVersion(versionAdapter.incrementVersion());
                            // also write this value to the project properties file
                            if (properties.getProperty(ProjectProperties.PROJECT_VERSION) == null) {
                            // here you should add it, and give it the new value
                                properties.addProperty(ProjectProperties.PROJECT_VERSION, tempVersion);
                            }
                            else {
                                properties.setProperty(ProjectProperties.PROJECT_VERSION, tempVersion);
                            }
                        } else {
                            // set the version in the version file
                            log.info("Set version: ");
                            versionAdapter.setCurrentVersion(buildDef.getVersion());
                            // and set the version in the project registry
                            if (properties.getProperty(ProjectProperties.PROJECT_VERSION) == null) {
                            // here you should add it, and give it the new value
                                properties.addProperty(ProjectProperties.PROJECT_VERSION, buildDef.getVersion());
                            }
                            else {
                                properties.setProperty(ProjectProperties.PROJECT_VERSION, buildDef.getVersion());
                            }
                        }
                        log.info("New version: "+buildDef.getVersion());
                        
                        // you now have to update the registry
                        properties.save();

                        buildDef.appendLogMessage("Version increment: OK")
                            .appendLogMessage("\n");
                    } catch (Throwable e) {
                        log.error(e.getMessage(), e);
                        buildDef.appendLogMessage(
                            "FAILED to increment version.  Message: ")
                            .appendLogMessage(e.getMessage())
                            .appendLogMessage("\n");
                        buildDef.setErrorFlag(true);
                    }
                }
                
                if (!buildDef.getErrorFlag()) {
                    tagProject(buildDef);
                }
                if (!buildDef.getErrorFlag()) { 
                    build(buildDef);
                }
                if (!buildDef.getErrorFlag()) {
                    publish(buildDef);
                }
            }
        }
        
        cleanup(buildDef);
    }
     
    //----------------------------------------------------------------------
    // checkout
    //----------------------------------------------------------------------    
    protected void checkout(BuildDefinition buildDef) { //AnthillProjectReleaseState releaseState) {
        RepositoryAdapter repositoryAdapter = 
            buildDef.getProject().getRepositoryAdapter();
            //releaseState.getProjectProperties().getRepositoryAdapter();
        try {

            // checkout the project from the repository
            log.info("Checkout: ");
            repositoryAdapter.checkout();

            buildDef.appendLogMessage("Check out: OK")
                .appendLogMessage("\n");
        } catch (Throwable e) {
            log.error(e.getMessage(), e);
            buildDef.appendLogMessage(
                "FAILED to perform check out.  Message: ")
                .appendLogMessage(e.getMessage()).appendLogMessage("\n");
            buildDef.setErrorFlag(true);
        }
    }
    
    //----------------------------------------------------------------------
    // determine whether the project requires a build.
    // basically check whether there are any revisions since the last build
    //----------------------------------------------------------------------    
    protected boolean checkForBuild(BuildDefinition buildDef) { //AnthillProjectReleaseState releaseState) {
        boolean requiresBuild = true;
        List revList = null;
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
//        String versionFilePath = properties.getLocalVersionFilePath();
        Date needRevisionsSinceDate = null;
        Date lastGoodDate = null;
        Date lastFailDate = null;

        if (properties.getLastBuildSucceededFlag()) {
            lastGoodDate = properties.getLastGoodBuildDate();
            needRevisionsSinceDate = lastGoodDate;
            log.debug("last good build date: " + lastGoodDate);
        } else {
            lastFailDate = properties.getLastBuildFailDate();
            log.debug("last build fail date: " + lastFailDate);
            if (lastFailDate != null) {
                needRevisionsSinceDate = lastFailDate;
            }
                // else it is null, so we have no dates, 
                // a build is required. 
        }
        
        try {
            if (needRevisionsSinceDate == null) {
                requiresBuild = true;
                log.info("Build required - no last build date exists");
            } else {
                revList = getRevisionsSince(needRevisionsSinceDate);
                buildDef.appendLogMessage("Got Revisions: OK\n");
              
                if (revList != null && !revList.isEmpty()) {
                    requiresBuild = true; // list isn't empty, so 
                    // we have revisions since that date...
                    if (properties.getLastBuildSucceededFlag()) {
                        buildDef.setRevisionList(revList);                        
                    } else {
                        if (properties.getLastGoodBuildDate() != null) {
                           buildDef.setRevisionList(
                                getRevisionsSince(
                                properties.getLastGoodBuildDate()));
                        }
                   }
/* 060902 - MBZ - no longer required because the RepositoryAdapter now filters out 
 *                all revisions made by the anthill user.
 *
                    // assume that all revisions are to the version file
                    // then check if there are any other revisions, 
                    // if so, then build is required
                    requiresBuild = false;
                    
                    // check whether there are any revisions to files other 
                    // than the version file
                    Revision revision = null;
                    int revListSize = revList.size();
//                    File versionFile = new File(versionFilePath);
//                    File revisionFile = null;
                    for (int i = 0; i < revListSize; i++) {
                        revision = (Revision)revList.get(i);
                        log.debug(revision);
                        if (revision.userName != null) {
//                        if (revision.fileName != null) {
//                            revisionFile = new File(revision.fileName);
                            if (revision.userName.equals("anthill")) {
//                            if (!revisionFile.equals(versionFile)) {
                                requiresBuild = true;
                                log.info("Build required - revisions to file " + 
                                    revision.fileName);
                                break;
                            }
                        }
                    }
*/
                } else {
                    requiresBuild = false;
                }
                
                if (!requiresBuild) {
                    log.info("Build NOT required");
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            buildDef.appendLogMessage(
                "FAILED to get revision list.  Message: ")
                .appendLogMessage(e.getMessage()).appendLogMessage("\n");
            buildDef.setErrorFlag(true);
        }
        
        buildDef.setRequiresBuildFlag(requiresBuild);
        return requiresBuild;
    }

    //----------------------------------------------------------------------
    // run the build script
    //----------------------------------------------------------------------    
    protected void build(BuildDefinition buildDef) {//AnthillProjectReleaseState releaseState) {
        StringBuffer logMessage = new StringBuffer();
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        
        String anthillUrl = properties.getAnthillUrl();
        String buildUrl = null;
        String pubUrl = properties.getPublishUrl();

        if ((pubUrl.equals("")) || (pubUrl == null)) {
            buildUrl = anthillUrl + "projects/" + getProjectName();
        }

        if ((pubUrl != null) && (!pubUrl.equals("")))  {
            buildUrl = pubUrl;
        }

        logMessage.append("Project Site: ")
            .append(buildUrl).append("\n\n")
            .append("Build Log: ")
            .append(buildUrl)
            .append("/buildLogs/")
            .append(getProjectName()).append("-")
            .append(buildDef.getVersion())
            .append("-build.log")
            .append("\n");

        try {
            // build the project using Ant and the build script w/ the
            // incremented version
            log.info("Build Project: ");
            
            buildProject(buildDef);

            buildDef.appendLogMessage("Build: OK\n");
            buildDef.appendLogMessage(logMessage.toString());            
        } catch (Throwable e) {
            log.error(e.getMessage(), e);
            
            buildDef.appendLogMessage(
                "FAILED to run build script.  Message: ")
                .appendLogMessage(e.getMessage()).appendLogMessage("\n")
                .appendLogMessage(logMessage.toString());
            
            buildDef.setErrorFlag(true);
        }
    }
       
    //----------------------------------------------------------------------
    // run the publish script
    //----------------------------------------------------------------------
    protected void publish(BuildDefinition buildDef) { //AnthillProjectReleaseState releaseState) {
        StringBuffer logMessage = new StringBuffer();
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        
        if (properties.getLocalPublishScriptFilePath() == null || 
            properties.getLocalPublishScriptFilePath().equals("")) {
            // do not do the publish
           buildDef.appendLogMessage("\nPublish: NOT REQUIRED\n");
            
        } else {
            // publish
            String anthillUrl = properties.getAnthillUrl();
            String buildUrl = null;
            String pubUrl = properties.getPublishUrl();

            if ((pubUrl.equals("")) || (pubUrl == null)) {
                buildUrl = anthillUrl + "projects/" + getProjectName();
            }

            if ((pubUrl != null) && (!pubUrl.equals("")))  {
                buildUrl = pubUrl;                        
            }

            logMessage.append("Publish Log: ")
                .append(buildUrl).append("/")
                .append("buildLogs/").append(getProjectName())
                .append("-").append(buildDef.getVersion())
                .append("-publish.log")
                .append("\n");

            try {
                log.info("Publish Project: ");
                publishProject(buildDef);

                buildDef.appendLogMessage("\nPublish: OK\n");
                buildDef.appendLogMessage(logMessage.toString());
            } catch (Throwable e) {
                log.error(e.getMessage(), e);
                buildDef.appendLogMessage(
                    "FAILED to run publish script.  Message: ")
                    .appendLogMessage(e.getMessage()).appendLogMessage("\n");
                buildDef.appendLogMessage(logMessage.toString());
                buildDef.setErrorFlag(true);
            }
        }
    }
        
    //--------------------------------------------------------------------------
    protected void tagProject(BuildDefinition buildDef) { 
        ProjectProperties properties = buildDef.getProject().getProperties();
//        String versionFilePath = properties.getLocalVersionFilePath();
        RepositoryAdapter repositoryAdapter = properties.getRepositoryAdapter();
        try {
/* 060702 - MBZ - moved to UrbancodeVersionAdapter
 *            
            // update the version file in the repository
            log.info("Update version file: ");
            repositoryAdapter.updateFile(versionFilePath);

            // commit the version file to the repository
            log.info("Commit version file: ");
            repositoryAdapter.commitFile(versionFilePath, null);
*/
            // tag the project with the new version in the repository
            log.info("Tag project: ");
            repositoryAdapter.tagProject(
                "v"+buildDef.getVersion());

            buildDef.appendLogMessage("Tag: OK").appendLogMessage("\n");            
        } catch (Throwable e) {
            buildDef.appendLogMessage("FAILED to tag project.  Message: ")
                .appendLogMessage(e.getMessage()).appendLogMessage("\n");
            buildDef.setErrorFlag(true);
        }
    }

    //----------------------------------------------------------------------
    // clean up 
    //----------------------------------------------------------------------   
    protected void cleanup(BuildDefinition buildDef) { //AnthillProjectReleaseState releaseState) {
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        RepositoryAdapter repositoryAdapter = properties.getRepositoryAdapter();
        StringBuffer logMessage = new StringBuffer();
        StringBuffer emailSubject = new StringBuffer();
//        String versionFilePath = properties.getLocalVersionFilePath();
        List revList = null;
        Date date = null;
        
/*        if (buildDef.getRequiresBuildFlag()) {

            // unlock the version file in the repository
            try {
                if (properties.getLockVersionFlag()) {
                    log.info("Unlock version file: ");
                    repositoryAdapter.unlockFile(versionFilePath);
                    logMessage.append("Unlock: OK").append("\n");
                }
            } catch (Exception e) {
                log.error("Version file ("+versionFilePath+") could not be unlocked.", e);
                logMessage.append("FAILED to unlock version file.  Message: ")
                    .append(e.getMessage()).append("\n");
            }
*/

            // erase the local project directory contents if not in debugging mode
            try {
                if (!log.isDebugEnabled()) {
                    log.info("Cleaning up local files: ");
                     File file = new File(repositoryAdapter.getLocalProjectDirName());
                    //.getWorkDirName());
                    FileRemover.removeFile(file);
                   // getRepositoryAdapter().revert(); // just removes working directory and files
                    logMessage.append("Removed temporary files: OK").append("\n");
                }
/*
                // erase the version file
                File vFile = new File(repositoryAdapter.getLocalProjectDirName(), 
                                      versionFilePath);
                vFile.delete();
*/
            } catch (Exception e) {
                log.error("Local project directory (" + 
                          repositoryAdapter.getLocalProjectDirName() + 
                          ") not erased.", e);
                logMessage.append("FAILED to remove temporary files.  Message: ")
                    .append(e.getMessage()).append("\n");
            }        
            
            // get the revisions and add them to the email
            revList = buildDef.getRevisionList();
            Revision rev = null;
            StringBuffer revLog = new StringBuffer();
            if (revList != null) {
                for (int i = 0; i < revList.size(); i++) {
                    rev = (Revision)revList.get(i);
                    revLog.append(rev.toString()).append("\n");
                }
            }
            logMessage.append(revLog.toString());

            // write the revision log
            StringBuffer revLogFileName = new StringBuffer();
            revLogFileName.append(properties.getBuildLogDir())
                .append(getProjectName())
                .append("-")
                .append(buildDef.getVersion())
                .append("-rev.log");
            try {
                FileWriter revLogFileWriter = 
                                    new FileWriter(revLogFileName.toString());
                revLogFileWriter.write(revLog.toString());
                revLogFileWriter.close();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            }

            // if the build has no errors, save the build date as the last good
            // build, otherwise save the fail date so builds won't continue
            // until there is a change to the project
  
            if (!buildDef.getErrorFlag()) {
                // here you have to check and see if the project actually built...
                if (buildDef.getRequiresBuildFlag() || buildDef.getForceBuildFlag()) {
                    this.properties.setLastGoodBuildDate(new Date());
                    this.properties.setLastBuildSucceededFlag(true);
                }
            } 
            else { // here there was an error, so I think it's safe to set 
                // the last failed date. 
                this.properties.setLastBuildFailDate(new Date());
                this.properties.setLastBuildSucceededFlag(false);
            }
            try {
                save();
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }            
        
        // only send email notification if a release took place and/or an
        // error occurred somewhere along the way
        if (buildDef.getForceBuildFlag() || 
            buildDef.getRequiresBuildFlag() || 
            (!buildDef.getRequiresBuildFlag() && 
             buildDef.getErrorFlag())) {
            
            // construct the email subject
            emailSubject.append(getProjectName());
            String newVersion = buildDef.getVersion();
            if (newVersion != null) {
                emailSubject.append(" ").append(newVersion);
            }
            emailSubject.append(" build ");
            if (buildDef.getErrorFlag()) {
                emailSubject.append("failed");
            } else {
                emailSubject.append("succeeded");
            }

            buildDef.appendLogMessage(logMessage.toString());

            // send the emails
            try {
                sendEmails(
                    buildDef, 
                    emailSubject.toString(), 
                    buildDef.getLogMessageBuffer().toString());
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        }
    }        

    //--------------------------------------------------------------------------
    protected void sendEmails(BuildDefinition buildDef, String subject, String content) //AnthillProjectReleaseState releaseState, String subject, String content) 
    throws Exception {
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        String mailhost = properties.getMailHost();
        String from = properties.getMailFrom();
        InternetAddress[] addressArray = null;
        
        // build the array of addresses 
        List emailList = new ArrayList();
        Map userMap = properties.getNotificationUserMap();
        if (userMap != null) {
            Iterator userItr = userMap.values().iterator();
            String email = null;
            while (userItr.hasNext()) {
                email = (String)userItr.next();
                emailList.add(email);
                log.debug("Will send email to: " + email);
            }
            addressArray = new InternetAddress[emailList.size()];
            for (int i = 0; i < emailList.size(); i++) {
                addressArray[i] = new InternetAddress((String)emailList.get(i));
            }

            // build the session
            Properties props = System.getProperties();
            props.put("mail.smtp.host", mailhost);
            Session session = Session.getDefaultInstance(props, null);
            session.setDebug(log.isDebugEnabled());

            log.debug("Mail.host: "+mailhost);

            // build the message
            Message msg = new MimeMessage(session);
            msg.setFrom(new InternetAddress(from));
            msg.setRecipients(Message.RecipientType.TO, addressArray);
            msg.setSubject(subject);
            msg.setText(content);
            msg.setHeader("X-Mailer", "Anthill");
            msg.setSentDate(new Date());

            log.debug("Email subject: "+subject);
            log.debug("Email content: "+content);

            // send it
            Transport.send(msg);

            log.debug("Email sent");
        }
    }
        
     //--------------------------------------------------------------------------
    protected void buildProject(BuildDefinition buildDef) //AnthillProjectReleaseState releaseState) 
    throws Exception {
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        ProjectTarget target = buildDef.getTarget();
        
        // collect extra parameters to be passed to the build script
        StringBuffer buildAntParams = appendAllKeyValues(properties.getBuildAntParams());
                        
        // build the Ant build command
        StringBuffer buildCmd = new StringBuffer().append(getJdkHome())
                .append(File.separator).append("bin")
                .append(File.separator).append("java")
                .append(" -classpath ")
                .append(anthillDefaults.getAntClassPath());
        
        buildCmd.append(" -DdeployDir="+properties.getProjectPublishDirName())
                .append(" -Dversion="+buildDef.getVersion())
                .append(" org.apache.tools.ant.Main")
                .append(" -buildfile ")
                .append(properties.getLocalBuildScriptFilePath())
                .append(" -logfile ")
                .append(properties.getBuildLogDir()).append(getProjectName())
                .append("-")
                .append(buildDef.getVersion())
                .append("-build.log");
        if (buildAntParams != null) {
            buildCmd.append(" ").append(buildAntParams.toString());
        }
        if (target != null && target.getName() != null) {
            buildCmd.append(" ").append(target.getName());
        }
 
        int exitCode = Execute.executeJavaCommand(buildCmd.toString());
 
        // handle errors
        if (exitCode != 0) {
            log.info("Query: " + buildCmd.toString());
            throw (new Exception("Failed to Build project."));
            
        }                
        
        log.info("Query: " + buildCmd.toString());
    }
        
    //--------------------------------------------------------------------------
    protected void publishProject(BuildDefinition buildDef) //AnthillProjectReleaseState releaseState) 
    throws Exception {
        ProjectProperties properties = buildDef.getProject().getProperties();//releaseState.getProjectProperties();
        ProjectTarget target = buildDef.getTarget();
        
        // collect extra parameters to be passed to the publish script
        StringBuffer pubAntParams = appendAllKeyValues(properties.getPubAntParams());
        
        // build the Ant publish command
        File publishDir = new File(properties.getProjectPublishDirName());
        String publishDirAbsPath = publishDir.getAbsolutePath();
        StringBuffer publishCmd = new StringBuffer().append(getJdkHome())
            .append(File.separator).append("bin")
            .append(File.separator).append("java")
            .append(" -classpath ")
            .append(anthillDefaults.getAntClassPath());

        publishCmd.append(" -DdeployDir="+publishDirAbsPath)
            .append(" -Dversion="+buildDef.getVersion())
            .append(" org.apache.tools.ant.Main")
            .append(" -buildfile ")
            .append(properties.getLocalPublishScriptFilePath())
            .append(" -logfile ")
            .append(properties.getBuildLogDir()).append(getProjectName())
            .append("-").append(buildDef.getVersion())
            .append("-publish.log");
        if (pubAntParams != null) {
            publishCmd.append(" ").append(pubAntParams.toString());
        }
        if (target != null && target.getName() != null) {
            publishCmd.append(" ").append(target.getName());
        }

        int exitCode = Execute.executeJavaCommand(publishCmd.toString());
 
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("Failed to Publish project."));
        }                
    }
    
    //--------------------------------------------------------------------------
    protected List getRevisionsSince(Date date) 
    throws Exception {
        return getRepositoryAdapter().getRevisionsSince(date);
    }
    
    //--------------------------------------------------------------------------
    protected String getJdkHome() {
        return anthillDefaults.getJdkHome();
    }
    
    //--------------------------------------------------------------------------
    public void doScheduleNameChanged(AnthillSchedule mySchedule) 
    throws Exception {
        properties.setScheduleName(mySchedule.getScheduleName());
        save();
    }
 
    //--------------------------------------------------------------------------
    private StringBuffer appendAllKeyValues(Map paramMap, String optionStr) {
        StringBuffer buffer = new StringBuffer();
        
        Iterator keys = paramMap.keySet().iterator();
        while (keys.hasNext()) {
            String key = (String)keys.next();
            String value = (String)paramMap.get(key);
            if (optionStr != null) {
                buffer.append(optionStr);
            }
            buffer.append(key).append("=").append(value).append(" ");
        }
        
        return buffer;
    }

    private StringBuffer appendAllKeyValues(List paramMap) {
        StringBuffer buffer = new StringBuffer();
        Iterator keys = paramMap.iterator();
       
        while (keys.hasNext()) {
//            String key = (String)keys.next();
//            String value = (String)paramMap.get(key);
            buffer.append(keys.next()).append(" ");
        }
        return buffer;
    }
}
