/*
 * @(#)AnthillSchedule.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import org.apache.log4j.Category;
import com.urbancode.anthill.util.*;
import com.urbancode.lib.registry.Registry;
import com.urbancode.lib.registry.RegistryEntry;
import java.io.*;
import java.util.*;
import java.text.*;
/**
 * @version $Revision: 1.10 $
 * @author 
 */
public class AnthillSchedule extends Thread {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(AnthillSchedule.class.getName());
    
    // suffix used by all schedule files
    static public final String SCHEDULE_FILE_SUFFIX = ".schedule";

    static public final String BUILD_INTERVAL_KEY   = "schedule.build.interval";
    
    static public final String START_INTERVAL_KEY       = "schedule.start.time";

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected Anthill anthill = null;
    protected Queue queue = null;
    protected String scheduleName = null;
    protected File file = null;
    protected List projectSet = new ArrayList();
    protected long lastBuildTime = 0l;
    protected long nextBuildTime = 0l;
    protected long buildInterval = 0l;
    protected long startInterval = 01;
    protected Registry registry = null;
    private boolean initialized = false;

    //--------------------------------------------------------------------------
    public AnthillSchedule(Anthill anthill, String name) 
    throws Exception {
        this.anthill = anthill;
        this.queue = anthill.getBuildQueue();
        this.scheduleName = name;
        // create the registry
        this.registry = Registry.createRegistry();
        // set default values
        setProperty(BUILD_INTERVAL_KEY, "0");
        setProperty(START_INTERVAL_KEY, "12:00");
        initialize();
    }

    //--------------------------------------------------------------------------
    public AnthillSchedule(Anthill anthill, String name, File file) 
    throws Exception {
        log.debug("Loading schedule: " + name);
        
        this.anthill = anthill;
        this.queue = anthill.getBuildQueue();
        this.scheduleName = name;
        this.file = file;
        // load the registry
        this.registry = Registry.loadFile(file);
        
        initialize();
    }

    //--------------------------------------------------------------------------
    public void setScheduleName(String name) {
        if (name == null) {
            throw new IllegalArgumentException("Project name cannot be null.");
        }
        String oldscheduleName = scheduleName;
        scheduleName = name;
        // if the name changed, then update Anthill
        if (!scheduleName.equals(oldscheduleName)) {
            anthill.doScheduleNameChange(oldscheduleName);
        }
    }
    
    //--------------------------------------------------------------------------
    public String getScheduleName() {
        return scheduleName;
    }
    
    //--------------------------------------------------------------------------
    public String getProperty(String name) {
        String value = null;
        
        // first try getting the value from the project properties
        value = registry.getKeyValue(name);
        
        log.debug("schedule " + scheduleName + " getProperty() " + name + " : " + value);
        
        return value;
    }
    
    //--------------------------------------------------------------------------
    public void setProperty(String name, String value) {
        registry.setKeyValue(name, value);
        initialized = false;
    }
    
    //--------------------------------------------------------------------------
    public void setBuildInterval(String key, String interval) {
        String buildInv = getProperty(BUILD_INTERVAL_KEY);
        if (!buildInv.equals(interval)) {
            setProperty(key, interval);            
            try {
                long temp = Long.parseLong(interval);
                buildInterval = temp * 1000 * 60; 
            } catch (NumberFormatException nfe) {
                log.error(nfe.getMessage(), nfe);
            } 
            this.interrupt();
        }else {
            setProperty(key, interval);
        }         
    }
        
     //-------------------------------------------------------------------------    
    public void setStartTime(String name, String value) {
        String startInv = getProperty(START_INTERVAL_KEY);
            if (!startInv.equals(value)) {
                this.setProperty(name, value);               
                this.interrupt();
            } else {
                this.setProperty(name, value);
            }
    }
        
        
    
    //--------------------------------------------------------------------------
    public long getTimeToNextBuild() {
        long time2nextBuild = 0l;
        
        // make sure we're initialized
        if (!initialized) {
            try {
                initialize();
            } catch (Exception e) {
            }
        }
        
        if (nextBuildTime > 0) {
            time2nextBuild = nextBuildTime - System.currentTimeMillis();
        } else if (nextBuildTime == 0) {
            try {
                time2nextBuild = getStartInterval();
            } catch (Exception e) {
            }
        }
        
        time2nextBuild = time2nextBuild / (1000 * 60);
                
        return time2nextBuild;
    }
    
    //--------------------------------------------------------------------------
    synchronized public void add(Buildable project) {
        log.debug("AnthillSchedule.add(): "+project);
        if (!projectSet.contains(project)) {
            projectSet.add(project);
        }
    }
    
    //--------------------------------------------------------------------------
    synchronized public void remove(Buildable project) {
        projectSet.remove(project);
    }
    
    //--------------------------------------------------------------------------
    synchronized public Iterator getProjectIterator() {
        return projectSet.iterator();
    }
    
    //--------------------------------------------------------------------------
    public void run() {
        log.debug("AnthillSchedule.run() name: "+getScheduleName());
        boolean resetStart = false;
        boolean resetBuild = false;
        
        do {
            if (startInterval > 0) {
               resetStart = false; 
               log.debug("Anthill schedule startInterval for: "+getScheduleName());
               log.debug("start interval equals :" +getProperty(START_INTERVAL_KEY));
                //sleep for start interval
                    try { 
                        sleep(startInterval);
                    } catch (InterruptedException i) {
                        log.error(i.getMessage(), i);
                        try {
                            this.initialize();
                        } catch (Exception e) {
                            log.error(e.getMessage(), e);
                        }
                        resetStart = true;                    
                }
            }
        }while (resetStart);
        
        // this way we can implement a schedule that does not actually 
        // build anything
        if (buildInterval > 0) {

            // start continuous loop
            while (true) {
                
                // synchronize on projectSet just in case anyone tries to add a 
                // project while we're doing this
                synchronized (projectSet) {
                    Iterator itr = projectSet.iterator();
                    Buildable project = null;
                    while (itr.hasNext()) {
                        // for each project in the project set, add the project to 
                        // the build queue
                        project = (Buildable)itr.next();
                        queue.add(project);
                    }
                }
                do {
                    resetBuild = false;
                    // now get ready to sleep
                    lastBuildTime = System.currentTimeMillis();
                    nextBuildTime = lastBuildTime + buildInterval;                   
                    try {
                        sleep(buildInterval);
                    } catch (InterruptedException ie) {
                        log.error(ie.getMessage(), ie);
                        resetBuild = true;
                    }
                }while (resetBuild);
            }
        }
    }

    //--------------------------------------------------------------------------
    public void save() 
    throws Exception {
        if (file != null) {
            if (!scheduleName.equals(FileUtils.getNameMinusExtension(file))) {
                // the file exists but the project name has been changed
                // we need to delete it so that we can save the schedule into 
                // a file whose name matches the name of the schedule
                file.delete();
                file = null;
                
                // create the new file
                createFile();
                
                // notify all the projects of the changed name
                notifyProjectsOfNameChange();
            } else {
                // file exists and is the same name as the project, 
                // save into the file
                try {
                    registry.updateRegistryFile();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                    throw e;
                }
            }
        } else {
            createFile();
        }
        
        // initialize the schedule with the changed properties
        initialize();
    }

    //--------------------------------------------------------------------------
    protected void createFile() 
    throws Exception {
        // we could have deleted the old file if the name of the schedule 
        // changed
        if (file == null) {
            // we need to create the file
            File confDir = anthill.getAnthillConfigDir();
            file = new File(confDir, scheduleName + SCHEDULE_FILE_SUFFIX);

            // try saving the registry into the file
            try {
                registry.setRegistryFile(file);
                registry.updateRegistryFile();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
                throw e;
            }
        }
    }
    
    //--------------------------------------------------------------------------
    protected void notifyProjectsOfNameChange() 
    throws Exception {
        Iterator itr = projectSet.iterator();
        Buildable project = null;
        while (itr.hasNext()) {
            project = (Buildable)itr.next();
            project.doScheduleNameChanged(this);
        }
    }
    
    //--------------------------------------------------------------------------
    protected void initialize() 
    throws Exception {
        long temp = 0l;
        boolean wasStopped = (buildInterval <= 0);

        String intervalStr = getProperty(BUILD_INTERVAL_KEY);
        if (intervalStr == null) {
            intervalStr = "0";
            setProperty(BUILD_INTERVAL_KEY, intervalStr);
        }
        String startIntervalStr = getProperty(START_INTERVAL_KEY);
        if (startIntervalStr == null) {
            startIntervalStr = "12:00";
            setProperty(START_INTERVAL_KEY, startIntervalStr);
        }
        
        startInterval = getStartInterval();
        
        try {
            temp = Long.parseLong(intervalStr);
        } catch (NumberFormatException nfe) {
            log.error(nfe.getMessage(), nfe);
            throw nfe;
        }
                   
        buildInterval = temp * 1000 * 60;
        
        initialized = true;
    }
    
    protected long getStartInterval()
    throws Exception {
        long now = 01;
        long start = 01;
        Date currentTime = new Date();
        SimpleDateFormat formatter = new SimpleDateFormat("H:m");
        
        try {
            String currentTimeStr = formatter.format(currentTime);
            Date currentTimeHrs = formatter.parse(currentTimeStr);
            Date startTimeHrs = formatter.parse(getProperty(START_INTERVAL_KEY));
            now = currentTimeHrs.getTime();
            start = startTimeHrs.getTime();
            if (currentTimeHrs.before(startTimeHrs)) {
                return (start - now);
            } else {
                return (1000L * 60L * 60L * 24L) - (now - start);
//                return 0; // starts immediately, or should we start tomorrow?
            }            
        } catch (ParseException pe) {
            log.error(pe.getMessage(), pe);
            throw pe;
        }
    }
    
}
