/*
 * @(#)ProjectProperties.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill;
import com.urbancode.anthill.adapter.*;
import com.urbancode.anthill.util.*;
import com.urbancode.lib.registry.*;
//import com.urbancode.anthill.adapter.VersionModifier;
import org.apache.log4j.Category;
import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.text.ParseException;
/**
 * @author Jason Buck
 */
public class ProjectProperties {
    
    //**************************************************************************
    // CLASS
    //**************************************************************************
    
    // Create Log4j category instance for logging
    static private Category log = 
        Category.getInstance(ProjectProperties.class.getName());
    
    static private final SimpleDateFormat DATE_FORMAT = 
        new SimpleDateFormat("MM/dd/yyyy HH:mm:ss");
    
    // suffix used by all project files
    static public final String ANTHILL_FILE_SUFFIX      = ".anthill";
    
    static public final String REGISTRY_ENTRY_KEY       = "anthill";
    static public final String PROJECT_NAME             = "project.name";
    static public final String BUILD_SCRIPT_PATH        = "build.script";
    static public final String PARAMS                   = "params";
    static public final String BUILD_SCRIPT_SYS_PARAMS  = "build";
    static public final String BUILD_SCRIPT_ANT_PARAMS  = "build.ant";
    static public final String BUILD_SCRIPT_PARAMS      = "build.params";
    static public final String PUBLISH_SCRIPT_PATH      = "publish.script";
    static public final String PUBLISH_SCRIPT_SYS_PARAMS= "publish";
    static public final String PUBLISH_SCRIPT_ANT_PARAMS= "publish.ant";
    static public final String PUBLISH_SCRIPT_PARAMS    = "publish.params";
//    static public final String BUILD_SYS_PARAM_NAME_KEY = "build.params.name";
    static public final String BUILD_ANT_PARAM_NAME_KEY = "build.ant.params.name";
//    static public final String PUB_SYS_PARAM_NAME_KEY   = "publish.params.name";
    static public final String PUB_ANT_PARAM_NAME_KEY   = "publish.ant.params.name";
    static public final String BUILD_ANT_PARAM_VALUE_KEY= "build.ant.params.value";
    static public final String PUB_ANT_PARAM_VALUE_KEY  = "publish.ant.params.value";
    static public final String BUILD_ANT_PARAMS         = "build.ant.params";
    static public final String PUB_ANT_PARAMS           = "publish.ant.params";
    
    static public final String BUILD_PUB_PARAM_VALUE_KEY= "value";
    static public final String PUBLISH_DIR_NAME         = "publish.dir";
    static public final String VERSION_ADAPTER          = "version.adapter";
    static public final String VERSION_FILE_PATH        = "version.file";
    static public final String VERSION_PROPERTIES       = "version";
    static public final String REPOSITORY_ADAPTER       = "repository.adapter";
    static public final String REPOSITORY_PROPERTIES    = "repository";
    static public final String SCHEDULE_KEY             = "schedule";
    static public final String MAIL_HOST_KEY            = "mail.host";
    static public final String MAIL_FROM_KEY            = "mail.from";
    static public final String USER_KEY                 = "users";
    static public final String LAST_GOOD_BUILD_DATE_KEY = "lastGoodBuildDate";
    static public final String LAST_BUILD_FAIL_DATE_KEY = "lastBuildFailDate";
    static public final String LAST_BUILD_SUCCEEDED_KEY = "lastBuildSucceeded";
    static public final String ANTHILL_URL_KEY          = "server";
    static public final String PUBLISH_URL_KEY          = "publish.url";
    static public final String LOCK_VERSION_KEY         = "lock.version.file";
    static public final String TARGETS_KEY              = "targets";
    static public final String DEFAULT_WORK_DIR         = "work.dir";
    static public final String WORK_DIR_KEY             = "repository.cvs.work.dir";
    static public final String DEPENDENCY_TARGET_KEY    = "dependency.target";
    static public final String DEPENDENCY_ARTIFACT_PROP_KEY = "dependency.artifact.property";

    static public final String REGISTRY_FORMAT_VERSION  = "registry.format.version";
    static public final String PROJECT_VERSION          = "project.version";
    static public final String CVS_BRANCH_KEY           = "repository.cvs.branch";
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    protected Anthill anthill = null;
    protected AnthillProject project = null;
    protected File file = null;
    protected Registry registry = null;
    protected RegistryEntry projectRegEntry = null;
    
    protected RepositoryAdapter repositoryAdapter = null;
    protected VersionAdapter versionAdapter = null;
    protected AnthillSchedule schedule = null;
    
    protected String repositoryAdapterName = null;
    protected String versionAdapterName = null;
    protected String scheduleName = null;
    protected String anthillUrl = null;
    protected String publishUrl = null;
    protected Map buildSysParams = new HashMap();
 //   protected Map buildAntParams = new HashMap();
    protected Map pubSysParams = new HashMap();
 //   protected Map pubAntParams = new HashMap();
    // added these
    protected List buildAntParams = new ArrayList();
    protected List pubAntParams = new ArrayList();
    
    
    protected Map notificationUserMap = new HashMap();
    protected String mailHost = null;
    protected String mailFrom = null;
    protected String lastGoodBuildDate = null;
    protected String lastBuildFailDate = null;
    protected String lastBuildSucceeded = null;
//    protected String lockVersion = null;
//    protected String versionFilePath = null;
    protected String buildScriptPath = null;
    protected String publishScriptPath = null;
    protected String publishDirName = null;

    protected String workDir = null;

    protected String dependencyTargetName = null;
    protected String dependencyArtifactProp = null;

    //--------------------------------------------------------------------------
    public ProjectProperties(Anthill anthill, AnthillProject project) {
        this.anthill = anthill;
        this.project = project;
        this.registry = Registry.createRegistry();
        this.projectRegEntry = registry.createChildRegistryEntry(ProjectProperties.REGISTRY_ENTRY_KEY);
        
        setupProperties(projectRegEntry);

        // I guess here you don't need to call the filter, 
        // but you do need to add an entry for the registry file version
        projectRegEntry.addKeyValue(REGISTRY_FORMAT_VERSION, "1.1"); 
        // so you'll have to change this value as you go, not a great way of
        // doing things, if anyone has any suggestions...
    }
    
    //--------------------------------------------------------------------------
    public ProjectProperties(Anthill anthill, AnthillProject project, File regFile)
    throws FileNotFoundException, IOException {
        this.anthill = anthill;
        this.project = project;
        this.registry = Registry.loadFile(regFile);
        File projectFile = new File(anthill.getAnthillProjectsDir()+File.separator +
                               project.getProjectName() + ".anthill");
        PropertyFilter0to1 pf = new PropertyFilter0to1(registry, projectFile);
        this.projectRegEntry = registry.getRegistryEntry(REGISTRY_ENTRY_KEY);        
        setupProperties(projectRegEntry);
    }
    
    //--------------------------------------------------------------------------
    public ProjectProperties(Anthill anthill, AnthillProject project, Registry registry) {
        this.anthill = anthill;
        this.project = project;
        this.registry = registry;
        File projectFile = new File(anthill.getAnthillProjectsDir()+File.separator +
                               project.getProjectName() + ".anthill");
        PropertyFilter0to1 pf = new PropertyFilter0to1(registry, projectFile);
        this.projectRegEntry = registry.getRegistryEntry(REGISTRY_ENTRY_KEY);        
        setupProperties(projectRegEntry);
    }
    
    //--------------------------------------------------------------------------
    /**
     * Returns a ProjectProperties instance which contains properties overridden
     * by the properties contained in the given RegistryEntry
     */
    public ProjectProperties mergeProperties(RegistryEntry regEntry) {
        ProjectProperties properties = 
            new ProjectProperties(anthill, project, registry);
        
        properties.setupProperties(projectRegEntry);
        properties.setupProperties(regEntry);
        
        return properties;
    }
    
    //--------------------------------------------------------------------------
    public RegistryEntry addProperty(String name, String value) {
        return projectRegEntry.addKeyValue(name, value);        
    }

    //--------------------------------------------------------------------------
    public String getProperty(String propName) {
        String propValue = projectRegEntry.getKeyValue(propName);
        
        // if there is no such property and the property,
        // try the anthill defaults
        if (propValue == null) {
            propValue = anthill.getAnthillConfig().getProperty(propName);
        }
        
        return propValue;
    }
    
    //--------------------------------------------------------------------------
    public RegistryEntry setProperty(String name, String value) {
        return projectRegEntry.setKeyValue(name, value);        
    }
    
    //--------------------------------------------------------------------------
    public RepositoryAdapter getRepositoryAdapter() {
        if (repositoryAdapter == null || 
            !repositoryAdapter.getClass().getName().equals(repositoryAdapterName)) {
            
            try {
                repositoryAdapter = RepositoryAdapterFactory.getRepositoryAdapter(
                    project, repositoryAdapterName);
            } catch (Exception e) {
                throw new IllegalStateException("exception while getting " +
                    repositoryAdapterName + " instance: " + e.getMessage());
            }
        }
        
        return repositoryAdapter;
    }
    
    //--------------------------------------------------------------------------
    public String getRepositoryAdapterName() {
        return repositoryAdapterName;
    }
    
    //--------------------------------------------------------------------------
    public void setRepositoryAdapterName(String repositoryAdapterName) {
        setProperty(REPOSITORY_ADAPTER, repositoryAdapterName);
        this.repositoryAdapterName = repositoryAdapterName;
        repositoryAdapter = null;
    }
    
    //--------------------------------------------------------------------------
    public VersionAdapter getVersionAdapter() {
        if (versionAdapter == null || 
            !versionAdapter.getClass().getName().equals(versionAdapterName)) {
            
            try {
                versionAdapter = VersionAdapterFactory.getVersionAdapter(
                    project, versionAdapterName);
            } catch (Exception e) {
                throw new IllegalStateException("exception while getting " +
                    versionAdapterName + " instance: " + e.getMessage());
            }
        }
        
        return versionAdapter;
    }
    
    //--------------------------------------------------------------------------
    public String getVersionAdapterName() {
        return versionAdapterName;
    }
    
    //--------------------------------------------------------------------------
    public void setVersionAdapterName(String versionAdapterName) {
        setProperty(VERSION_ADAPTER, versionAdapterName);
        this.versionAdapterName = versionAdapterName;
        versionAdapter = null;
    }
    
    //--------------------------------------------------------------------------
    public AnthillSchedule getSchedule() {
/*
        if (schedule == null ||
            !schedule.getScheduleName().equals(scheduleName)) {

            if (scheduleName != null) {
                schedule = anthill.getSchedule(scheduleName);
            }
            if (schedule != null) {
                schedule.add(project);
            }
        }
*/        
        return schedule;
    }
    
    //--------------------------------------------------------------------------
    public void setScheduleName(String scheduleName) {
        setProperty(SCHEDULE_KEY, scheduleName);
        this.scheduleName = scheduleName;
    }
    
    //--------------------------------------------------------------------------
    public String getAnthillUrl() {
        return anthillUrl;
    }
    
    //--------------------------------------------------------------------------
    public void setAnthillUrl(String anthillUrl) {
        setProperty(ANTHILL_URL_KEY, anthillUrl);
        this.anthillUrl = anthillUrl;
    }
    
    //--------------------------------------------------------------------------
    public String getPublishUrl() {
        return publishUrl;
    }

    //--------------------------------------------------------------------------
    public void setPublishUrl(String publishUrl) {
        setProperty(PUBLISH_URL_KEY, publishUrl);
        this.publishUrl = publishUrl;
    }
    
    //--------------------------------------------------------------------------
    public String getMailHost() {
//        return mailHost;
        return anthill.getAnthillConfig().getProperty(MAIL_HOST_KEY);
    }
    
    //--------------------------------------------------------------------------
    public void setMailHost(String mailHost) {
//        setProperty(MAIL_HOST_KEY, mailHost);
        anthill.getAnthillConfig().setProperty(MAIL_HOST_KEY, mailHost);
        this.mailHost = mailHost;
    }

    //--------------------------------------------------------------------------
    public String getMailFrom() {
//        return mailFrom;
        return anthill.getAnthillConfig().getProperty(MAIL_FROM_KEY);
    }
    
    //--------------------------------------------------------------------------
    public void setMailFrom(String mailFrom) {
//        setProperty(MAIL_FROM_KEY, mailFrom);
        anthill.getAnthillConfig().setProperty(MAIL_FROM_KEY, mailFrom);
        this.mailFrom = mailFrom;
    }
    
    //--------------------------------------------------------------------------
    public Date getLastGoodBuildDate() {
        Date date = null;
        if (lastGoodBuildDate != null) {
            try {
                date = DATE_FORMAT.parse(lastGoodBuildDate);
            } catch (ParseException pe) {
                log.error("exception parsing " + LAST_GOOD_BUILD_DATE_KEY, pe);
            }
        }
        
        return date;
    }
    
    //--------------------------------------------------------------------------
    public void setLastGoodBuildDate(Date date) {
        String dateStr = DATE_FORMAT.format(date);
        setProperty(LAST_GOOD_BUILD_DATE_KEY, dateStr);
        lastGoodBuildDate = dateStr;
    }
    
    //--------------------------------------------------------------------------
    public Date getLastBuildFailDate() {
        Date date = null;
        if (lastBuildFailDate != null) {
            try {
                date = DATE_FORMAT.parse(lastBuildFailDate);
            } catch (ParseException pe) {
                log.error("exception parsing " + LAST_BUILD_FAIL_DATE_KEY, pe);
            }
        }
        
        return date;
    }
    
    //--------------------------------------------------------------------------
    public void setLastBuildFailDate(Date date) {
        String dateStr = DATE_FORMAT.format(date);
        setProperty(LAST_BUILD_FAIL_DATE_KEY, dateStr);
        lastBuildFailDate = dateStr;
    }
    
    //--------------------------------------------------------------------------
    public boolean getLastBuildSucceededFlag() {
        return (lastBuildSucceeded == null || 
                lastBuildSucceeded.equals("true"));
    }
    
    //--------------------------------------------------------------------------
    public void setLastBuildSucceededFlag(boolean succeededFlag) {
        String flagStr = succeededFlag ? "true" : "false";
        setProperty(LAST_BUILD_SUCCEEDED_KEY, flagStr);
        this.lastBuildSucceeded = flagStr;
    }
    
/*
    //--------------------------------------------------------------------------
    public boolean getLockVersionFlag() {
        return ((lockVersion.equals("") ||
                (lockVersion.equalsIgnoreCase("On"))));
    }
*/
/*
    //--------------------------------------------------------------------------
    public String getVersionFilePath() {
        return versionFilePath;
    }
*/
/*    
    //--------------------------------------------------------------------------
    **
     * Returns the path within the working directory to the local copy of the 
     * version file.
     *
     * @return path to the local copy of the version file.
     *
    public String getLocalVersionFilePath() {        
        String localVersionFilePath =  
            project.getRepositoryAdapter().getLocalProjectDirName() + 
            File.separator + versionFilePath;
        
        return localVersionFilePath;
    }
*/
    //--------------------------------------------------------------------------
    public String getBuildScriptPath() {
        return buildScriptPath;
    }
    
    //--------------------------------------------------------------------------
    /**
     * Returns the path to the local copy of the build script.
     *
     * @return path to the local copy of the build script.
     */
    public String getLocalBuildScriptFilePath() {
        String localBuildScriptFilePath =  
            project.getRepositoryAdapter().getLocalProjectDirName() + 
            File.separator + buildScriptPath;
        
        return localBuildScriptFilePath;
    }
    
    //--------------------------------------------------------------------------
    public String getPublishScriptPath() {
        return publishScriptPath;
    }
    
    //--------------------------------------------------------------------------
    /**
     * Returns the path to the local copy of the publish script.
     *
     * @return path to the local copy of the publish script.
     */
    public String getLocalPublishScriptFilePath() {
        String localPublishScriptFilePath = null;
        log.debug("publishScriptPath =" + publishScriptPath); 
        if (publishScriptPath !=null &&
            !publishScriptPath.equals("")) {
                
                localPublishScriptFilePath =
                    project.getRepositoryAdapter().getLocalProjectDirName() + 
                    File.separator + publishScriptPath;
        }
        
        return localPublishScriptFilePath;
    }
    
    //--------------------------------------------------------------------------
    public File getAnthillDefaultPublishDir() {
        return anthill.getAnthillDefaultPublishDir();
    }
    
    //--------------------------------------------------------------------------
    public String getPublishDir() {
        return publishDirName;
    }
    
    //--------------------------------------------------------------------------
    public String getProjectPublishDirName() {
        String dirName = publishDirName;
        if (dirName == null || dirName.equals("")) {
            // the publish directory is empty
            // assume the publish dir location is the project name 
            dirName = project.getProjectName();

            File publishRootDir = anthill.getAnthillDefaultPublishDir();            
            File properPath = FileUtils.makeProperPath(
                publishRootDir, dirName);
            dirName = properPath.getAbsolutePath();
        }
        
        return dirName;
    }        

    //--------------------------------------------------------------------------
    public String getBuildLogDir() {
        // compute the Ant log file base name
        String buildLogDirName = getProjectPublishDirName() + File.separator + 
                                 "buildLogs" + File.separator;
        // make sure the log dir exists
        File buildLogDir = new File(buildLogDirName);
        if (!buildLogDir.exists()) {
            buildLogDir.mkdirs();
        }
        return buildLogDirName;
    }
    
/*
    //--------------------------------------------------------------------------
    public String getWorkDir() {
        return workDir;
    }
*/
    //--------------------------------------------------------------------------
    public String getDependencyTargetName() {
        if (dependencyTargetName == null) {
            return "dependency";
        }
        return dependencyTargetName;
    }
    
    //--------------------------------------------------------------------------
    public String getDependencyArtifactProperty() {
        if (dependencyArtifactProp == null) {
            return "dependency.artifact.dir";
        }
        return dependencyArtifactProp;
    }
    
    //--------------------------------------------------------------------------
    public void addBuildAntParam(String value) {
        RegistryEntry regEntry = addProperty(BUILD_ANT_PARAM_VALUE_KEY, value);
     //   regEntry.addKeyValue(BUILD_PUB_PARAM_VALUE_KEY, value);
     //   buildAntParams.put(value, value);
        buildAntParams.add(value);
        log.debug("adding build param: " + value);
    }
   
    //--------------------------------------------------------------------------
    public void addPubAntParam(String value) {
        RegistryEntry regEntry = addProperty(PUB_ANT_PARAM_VALUE_KEY, value);
        //regEntry.addKeyValue(BUILD_PUB_PARAM_VALUE_KEY, value);
        //pubAntParams.put(value, value);
        pubAntParams.add(value);
    }
    
    //--------------------------------------------------------------------------

    public List getBuildAntParams() {
        return buildAntParams;
    }
    
    //--------------------------------------------------------------------------

    public List getPubAntParams() {
        return pubAntParams;
    }
    
    //--------------------------------------------------------------------------
    public void removeBuildParamSysProperty(String key) {
        removeParamProperty(
            projectRegEntry.getChildRegistryEntry(BUILD_SCRIPT_SYS_PARAMS), 
            key);
       // buildSysParams.remove(key);
    }    
    //--------------------------------------------------------------------------
    public void removeBuildParamAntProperty(String key) {
        removeParamProperty(
            projectRegEntry.getChildRegistryEntry(BUILD_SCRIPT_ANT_PARAMS), 
            key);
        log.debug("trying to remove key: " + key);
        buildAntParams.remove(key);
    }    
    //--------------------------------------------------------------------------
    public void removePublishParamSysProperty(String key) {
        removeParamProperty(
            projectRegEntry.getChildRegistryEntry(PUBLISH_SCRIPT_SYS_PARAMS), 
            key);
      //  pubSysParams.remove(key);
    }   
    //--------------------------------------------------------------------------
    public void removePublishParamAntProperty(String key) {
        removeParamProperty(
            projectRegEntry.getChildRegistryEntry(PUBLISH_SCRIPT_ANT_PARAMS),
            key);
        log.debug("Trying to remove key: " + key);
        pubAntParams.remove(key);
    }    

    //--------------------------------------------------------------------------
    public void addNotificationUser(String userName, String email) {
        setProperty(userName, email);
        notificationUserMap.put(userName, email);
    }
    
    //--------------------------------------------------------------------------
    public Map getNotificationUserMap() {
        return notificationUserMap;
    }
    
    //--------------------------------------------------------------------------
    public void removeNotificationUser(String userName) {
        RegistryEntry regEntry = 
            projectRegEntry.getChildRegistryEntry(USER_KEY);
        if (regEntry != null) {
            regEntry.removeKey(userName);
        }
        notificationUserMap.remove(userName);
    }
    
    //--------------------------------------------------------------------------
    public void save() 
    throws IOException {
        // switch the schedules
        if (scheduleName != null) {
            AnthillSchedule newSchedule = anthill.getSchedule(scheduleName);
            if (!newSchedule.equals(schedule)) {
                // the new schedule is different from the old schedule
                // try to remove the project from the old schedule and 
                // add it to the new schedule
                if (schedule != null) {
                    // the old schedule is not null and different from the 
                    // new schedule, remove this project from the old schedule
                    schedule.remove(project);
                }
                newSchedule.add(project);
            }
            schedule = newSchedule;
        } else {
            // the new schedule is null, don't need to add this project to 
            // the new schedule
            if (schedule != null) {
                // the old schedule is not null and different from the 
                // new schedule, remove this project from the old schedule
                schedule.remove(project);
            }
            schedule = null;
        }
        
        if (file != null) {
            if (!project.getProjectName().equals(FileUtils.getNameMinusExtension(file))) {
                // the file exists but the project name has been changed
                // we need to delete it so that we can save the project into 
                // a file whose name matches the name of the project
                file.delete();
                file = null;
            } else {
                // file exists and is the same name as the project, 
                // save into the file
                try {
                    registry.updateRegistryFile();
                } catch (IOException e) {
                    log.error(e.getMessage(), e);
                    throw e;
                }
            }
        } 
        
        // we could have deleted the file above if the name of the project 
        // changed
        if (file == null) {
            // we need to create the file
            File projectsDir = anthill.getAnthillProjectsDir();
            file = new File(projectsDir, project.getProjectName() + ANTHILL_FILE_SUFFIX);

            // try saving the registry into the file
            try {
                registry.setRegistryFile(file);
                registry.updateRegistryFile();
            } catch (IOException e) {
                log.error(e.getMessage(), e);
                throw e;
            }
        }
        log.debug(project.getProjectName() + " properties saved");
        
        setupProperties(projectRegEntry);
    }
    
    //--------------------------------------------------------------------------
    protected void setupProperties(RegistryEntry regEntry) {
        // setup Repository Adapter Name
        repositoryAdapterName = getPropValue(
            regEntry, REPOSITORY_ADAPTER, repositoryAdapterName);
        repositoryAdapter = null;
        
        // setup Version Adapter Name
        versionAdapterName = getPropValue(
            regEntry, VERSION_ADAPTER, versionAdapterName);
        versionAdapter = null;
        
        // setup Schedule Name
        scheduleName = getPropValue(
            regEntry, SCHEDULE_KEY, scheduleName);
        if (scheduleName != null) {
            schedule = anthill.getSchedule(scheduleName);
            schedule.add(project);
        }
        
        // setup Anthill Url
        anthillUrl = getPropValue(
            regEntry, ANTHILL_URL_KEY, anthillUrl);
        
        // setup Publish Url
        publishUrl = getPropValue(
            regEntry, PUBLISH_URL_KEY, publishUrl);
        
        // setup Mail Host
        mailHost = getPropValue(
            regEntry, MAIL_HOST_KEY, mailHost);
        
        // setup Mail From
        mailFrom = getPropValue(
            regEntry, MAIL_FROM_KEY, mailFrom);
        
        // setup Last Build Succeeded
        lastBuildSucceeded = getPropValue(
            regEntry, LAST_BUILD_SUCCEEDED_KEY, lastBuildSucceeded);
        
        // setup Last Good Build Date
        lastGoodBuildDate = getPropValue(
            regEntry, LAST_GOOD_BUILD_DATE_KEY, lastGoodBuildDate);
        
        // setup Last Build Fail Date
        lastBuildFailDate = getPropValue(
            regEntry, LAST_BUILD_FAIL_DATE_KEY, lastBuildFailDate);
        
        // setup Lock Version
//        lockVersion = getPropValue(
//            regEntry, LOCK_VERSION_KEY, lockVersion);
        
        // setup Version File Path
//        versionFilePath = getPropValue(
//            regEntry, VERSION_FILE_PATH, versionFilePath);

        // setup Build Script Path
        buildScriptPath = getPropValue(
            regEntry, BUILD_SCRIPT_PATH, buildScriptPath);
        
        // setup Publish Script Path
        publishScriptPath = getPropValue(
            regEntry, PUBLISH_SCRIPT_PATH, publishScriptPath);

        // setup Publish Directory Name
        publishDirName = getPropValue(
            regEntry, PUBLISH_DIR_NAME, publishDirName);
            
        // setup Work Directory
        workDir = getPropValue(
            regEntry, WORK_DIR_KEY, workDir);
      
        // setup Dependency Target Name
        dependencyTargetName = getPropValue(
            regEntry, DEPENDENCY_TARGET_KEY, dependencyTargetName);
        
        // setup Dependency Artifact Property
        dependencyArtifactProp = getPropValue(
            regEntry, DEPENDENCY_ARTIFACT_PROP_KEY, dependencyArtifactProp);
        
        // setting up the build and publish params will now 
        // be getting an iterator for the RegistryEntry
        // so for now, just comment these out...
        // GETS YOU TO build.ant
        buildAntParams.clear();
        buildAntParams.addAll(buildParamList(regEntry.getChildRegistryEntry(BUILD_SCRIPT_ANT_PARAMS)));
        
        pubAntParams.clear();
        pubAntParams.addAll(buildParamList(regEntry.getChildRegistryEntry(PUBLISH_SCRIPT_ANT_PARAMS)));
/*        
        // setup Build Ant Parameters
        Map tempBuildAntParams = buildParamMap(
            regEntry.getChildRegistryEntry(BUILD_SCRIPT_ANT_PARAMS));
        buildAntParams.putAll(tempBuildAntParams); // override params
        
        // setup Publish Ant Parameters
        Map tempPubAntParams = buildParamMap(
            regEntry.getChildRegistryEntry(PUBLISH_SCRIPT_ANT_PARAMS));
        pubAntParams.putAll(tempPubAntParams); // override params
*/
        
        // setup Notification Users
        Map tempNotificationUserMap = buildUserMap(
            regEntry.getChildRegistryEntry(USER_KEY));
        notificationUserMap.putAll(tempNotificationUserMap); // override
    }
    
    //--------------------------------------------------------------------------
    protected String getPropValue(RegistryEntry regEntry, String propName, String curPropValue) {
        // first try getting the value from the given RegistryEntry
        String propValue = regEntry.getKeyValue(propName);
        
        // if there is no such property and the property currently has no value, 
        // try the anthill defaults
        if (propValue == null) {
            if (curPropValue == null) {
                propValue = anthill.getAnthillConfig().getProperty(propName);
            } else {
                // just keep the current value
                propValue = curPropValue;
            }
        }
        
        return propValue;
    }
    
    //--------------------------------------------------------------------------
    protected Map buildUserMap(RegistryEntry userEntry) {
        Map userMap = new HashMap();
        if (userEntry != null) {
            Iterator userNameItr = userEntry.getKeyIterator();
            String userName = null;
            if (userNameItr != null) {
                while (userNameItr.hasNext()) {
                    userName = (String)userNameItr.next();
                    userMap.put(userName, userEntry.getKeyValue(userName));
                }
            }
        }
        
        return userMap;
    }
    
    //--------------------------------------------------------------------------
    protected void removeParamProperty(RegistryEntry paramsReg, String key) {
        if (paramsReg != null) {
            Iterator paramsRegItr = paramsReg.getChildRegistryEntries(PARAMS);
            while (paramsRegItr.hasNext()) {
                RegistryEntry keyReg = (RegistryEntry)paramsRegItr.next();
                if (key.equals(keyReg.getKeyValue("value"))) {
                    paramsReg.removeChildRegistryEntry(keyReg);
                    break;
                }
            }
        }
    }
    
    //--------------------------------------------------------------------------
    protected void oldRemoveParamProperty(RegistryEntry paramsReg, String key) {
        if (paramsReg != null) {
            Iterator paramsRegItr = paramsReg.getChildRegistryEntries(PARAMS);
            while (paramsRegItr.hasNext()) {
                RegistryEntry keyReg = (RegistryEntry)paramsRegItr.next();
                if (key.equals(keyReg.getKeyValue("name"))) {
                    paramsReg.removeChildRegistryEntry(keyReg);
                    break;
                }
            }
        }
    }
    
    //--------------------------------------------------------------------------
    protected Map buildParamMap(RegistryEntry paramEntry) {
        Map paramMap = new HashMap();
        if (paramEntry != null) {
            Iterator regEntryItr = paramEntry.getChildRegistryEntries(PARAMS);
            if (regEntryItr != null) {
                while (regEntryItr.hasNext()) {
                    RegistryEntry regEntry = (RegistryEntry)regEntryItr.next();
                    String paramName = regEntry.getKeyValue("name");
                    String paramValue = regEntry.getKeyValue("value");
                    if (paramName != null && paramValue != null) {
                        paramMap.put(paramName, paramValue);
                    }
                }
            }
        }
        
        return paramMap;
    }
    
    protected List buildParamList(RegistryEntry paramEntry) {
        List paramList = new ArrayList();
        if (paramEntry != null) {
            Iterator paramItr = paramEntry.getChildRegistryEntries(PARAMS);
            if (paramItr != null) {
                while (paramItr.hasNext()) {
                    RegistryEntry regEntry = (RegistryEntry)paramItr.next();
                    String paramValue = regEntry.getKeyValue("value");
                    paramList.add(paramValue);
                    log.debug("Adding param: " + paramValue);
                }
            }
            else {
                log.error("param Iterator was null!");
            }
        }
        else {
            log.error("paramEntry was null!");
        }
        return paramList;
    }
}
