/*
 * @(#)CVSRepositoryAdapter.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.adapter;

import org.apache.log4j.Category;
import com.urbancode.anthill.util.Execute;
import com.urbancode.anthill.util.StreamPumper;
import com.urbancode.anthill.AnthillProject;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.ArrayList;
import java.util.Date;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.util.TimeZone;
/**
 * <p>
 * An implementation of <code>RepositoryAdapter</code> that interfaces with
 * a command-line CVS client to work with a CVS repository.</p>
 * <p>
 * The following extra properties must be specified for this adapter in the
 * .anthill file:
 * <ul>
 *   <li>CVS_MODULE - the module name to work with for the project
 *   <li>CVS_ROOT - the CVSROOT to connect to the CVS repository
 *                  eg. :pserver:username@repository URL:repository location
 * </ul></p>
 * <p>
 * <b>IMPORTANT: This implementation does not log in, so you must have logged
 * into the CVS repository you are using from the machine you are running this
 * on with the same user. There seems to be some complication on passing
 * information to a <code>Process</code> once it is started in Java.</b></p>
 * 
 * @version $Revision: 1.32 $
 * @author  Ryan Smith
 */
public class CVSRepositoryAdapter extends RepositoryAdapter {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(CVSRepositoryAdapter.class.getName());
    // Create Log4j category instance for logging
    static private Category streamLog = Category.getInstance(CVSRepositoryAdapter.class.getName()+"Stream");
    // CVS project property key
    static public final String CVS_MODULE_KEY = "repository.cvs.module";
    // CVS root property key
    static public final String CVS_ROOT_KEY = "repository.cvs.root";
    // CVS branch property key
    static public final String CVS_BRANCH_KEY = "repository.cvs.branch";
    static public final String CVS_ANTHILL_USER_KEY = "repository.cvs.anthill.user";    
    // date format understood by CVS commands
    private static final SimpleDateFormat CVS_IN_DATE = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    // date format returned by CVS commands
    private static final SimpleDateFormat CVS_OUT_DATE = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
    
    //==========================================================================
    // TOKEN USED FOR PARSING LOG COMMAND RESULTS
    //==========================================================================
    // Token separating logs info different files
    private static final String FILE_DELIM_TOKEN = "=============================================================================";    
    // Token specifying the RCS file name
//    private static final String RCS_FILE_TOKEN = "RCS file: ";
    // Token specifying the Working file name
    private static final String WORKING_FILE_TOKEN = "Working file: ";
    // Token separating the revisions of a file
    private static final String REV_DELIM_TOKEN = "----------------------------";
    // Token specifying the revision number.  We use this token to signal the 
    // start of a revision.
    private static final String REV_NUMB_TOKEN = "revision ";
    // Token specifying the date of a revision
    private static final String REV_DATE_TOKEN = "date:";
    private static final String REV_REVISIONS_TOKEN = "total revisions:";
    
    // ADDED CODE (SPB) 
    private static final String SYMBOLIC_LIST_TOKEN = "symbolic names:";
    private static final String KEYWORD_SUBSTITUTIONS_TOKEN = "keyword substitution:";
    // END ADDED CODE (SPB)
    
    // Token specifying the author of the revision
//    private static final String REV_AUTHOR_TOKEN = "author:";
    // Token specifying the state of the revised file
//    private static final String REV_STATE_TOKEN = "state:";
    // Value for the state.  Soecifies that the revision deleted the file
    private static final String REV_STATE_DEAD_TOKEN = "dead";
    // Token specifying the number of lines touched by the revision
//    private static final String REV_LINES_TOKEN = "lines: ";
    private static final String NEW_LINE = System.getProperty("line.separator");
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    
    /**
     * Create a new CVSRepositoryAdapter
     */
    public CVSRepositoryAdapter() {
    }

    /**
     * Calculates the branch that this project is on, if any.
     */
    protected void calculateBranchName() {
        branchName = project.getProperties().getProperty(CVS_BRANCH_KEY);
        if (branchName.equals("")) {
            branchName = null;
        }
    }
    
    /**
     * Log into CVS.
     */
    public void login() throws Exception {
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        log.info("Assuming that login to "+cvsroot+" exists in .cvspass file.");
        
        // HACK - we are assuming the user is logged into the CVS client
        setLoggedIn();
    }

    /**
     * Log out of CVS.
     */
    public void logout() throws Exception {
        setLoggedOut();
    }
    
    /**
     * Update a file from CVS.
     *
     * @param file  the name of the file to update
     */
    public void updateFile(String file) throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        log.info("Updating file: "+file);
                 
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        
        String commands[] = {"cvs",
                             "-d",
                             cvsroot,
                             "update",
                             file};
                             
        int exitCode = Execute.executeCommand(commands, 
                               "update", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs update failed"));
        }                
    }
    
    /**
     * Commit a file to CVS.
     *
     * @param file  the name of the file to commit
     */
    public void commitFile(String file, String msg) throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        log.info("Commiting file: "+file);
                 
        if (msg == null || msg.length() == 0) {
            msg = "\"Committed by Anthill\"";
        }
        
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        
        String commands[] = {"cvs",
                             "-d",
                             cvsroot,
                             "commit",
                             "-m",
                             msg,
                             file};
                             
        int exitCode = Execute.executeCommand(commands, 
                               "commit", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs commit failed"));
        }                
    }
    
    /**
     * Tag a project in CVS.
     *
     * @param tag  the value of the tag to tag the project with
     */
    public void tagProject(String tag) throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        if (tag == null || tag.length() == 0) {
            throw (new Exception("No tag specified"));
        }
        
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        tag = tag.replace('$','_').replace(',','_')
                 .replace('.','_').replace(':','_')
                 .replace(';','_').replace('@','_');
        
        log.info("Tagging entire project with tag: "+tag);
                 
        // replace
        String commands[] = {"cvs",
                             "-d",
                             cvsroot,
                             "tag",
                             tag,
                             getLocalProjectDirName()};
                             
        int exitCode = Execute.executeCommand(commands, 
                               "tag", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs tag failed"));
        }                
    }
    
    /**
     * Unlock a file with CVS.
     *
     * @param file  the name of the file to unlock
     */
    public void unlockFile(String file) throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        log.info("Unlocking file: "+file);
        
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        
        String commands[] = {"cvs",
                             "-d",
                             cvsroot,
                             "admin",
                             "-u",
                             file};
                             
        int exitCode = Execute.executeCommand(commands, 
                               "unlock", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs unlock failed"));
        }                
    }
    
    /**
     * Lock a file with CVS.
     *
     * @param file  the name of the file to lock
     */
    public void lockFile(String file) throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        log.info("Locking file: "+file);
        
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        
        String commands[] = {"cvs",
                             "-d",
                             cvsroot,
                             "admin",
                             "-l",
                             file};
                             
        int exitCode = Execute.executeCommand(commands, 
                               "lock", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs lock failed"));
        }                
    }
    
    /**
     * Checkout the project from CVS.
     */
    public void checkout() throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
 
        // workingDirectory is the inherited local working directory
        String module = project.getProperties().getProperty(CVS_MODULE_KEY);
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        String branch = getBranchName();
        
        log.info("Checking out module: "+module);
        log.info("Checking out branch: "+branch);
        
        StringBuffer command = new StringBuffer();
        command.append("cvs ")
               .append("-d ").append(cvsroot)
               .append(" checkout");
        
        String workDirName = getWorkDirName();
        if (workDirName == null) {
           throw new IllegalStateException("work dir property not set ");
        }

        // if the work directory is empty, then don't include it
        if (workDirName != null && !workDirName.equals("")) {
            command.append(" -d ").append(workDirName);
        }
        
        // if there is not a branch, then don't include it
        if (branch != null) {
            command.append(" -r ").append(branch);
        }
        // do include the module
        // the -N option does not shorten the path when -d is specified
        command.append(" -N ").append(module);
        
        log.debug("CheckoutCommand: "+command.toString());
        
        int exitCode = Execute.executeCommand(command.toString(), 
                               "checkout", 
                               log.isDebugEnabled());
        // handle errors
        if (exitCode != 0) {
            throw (new Exception("cvs checkout failed"));
        }                
    }

    /*
     * REVISIT
     * CVS on unix won't take a multi-level directory for the
     * "checkout -d" option.  This is a hack which uses the branch 
     * name in place of the working dir so branches won't collide
     * with each other.
     */
    protected void calculateWorkDirName() 
    throws Exception {
        super.calculateWorkDirName();
        String branchName = getBranchName();
        if (branchName != null) {
            workDirName = branchName;
        }
    }
    
    protected void calculateLocalProjectDirName() {
        // make sure that the work dir name has been set
        if (getWorkDirName() == null) {
            throw new IllegalStateException(
                "CVSRepositoryAdapter.calculateLocalProjectDirName() called " +
                "before the work directory name was set.");
        }
        
        String cvsModule = project.getProperties().getProperty(CVS_MODULE_KEY);
        if (log.isDebugEnabled()) {
            log.debug("cvsModule: "+cvsModule);
        }
        
        // TODO - I don't think that we need to lokk for this
        // the local project directory will always be the 
        // work dir plus the module
        
        /*
        // look for the sub module name 
        String moduleName = null;
        String subModuleName = null;
        int index = 0;
        if ((index = cvsModule.indexOf("/")) > 0) {
            subModuleName = cvsModule.substring(index + 1);
        } else if ((index = cvsModule.indexOf("\\")) > 0) {
            subModuleName = cvsModule.substring(index + 1);
        }
        if (log.isDebugEnabled()) {
            log.debug("subModuleName: "+subModuleName);
        }
        
        // set the localProjectDirName 
        String projectName = getAnthillProject().getProjectName();
        log.debug("projectName: "+projectName);
        */
        /*
         * REVISIT
         * CVS on unix won't take a multi-level directory for the
         * "checkout -d" option.  This is a hack which uses the branch 
         * name in place of the working dir so branches won't collide
         * with each other.
         */        
        String branchName = getBranchName();        
        String tempWorkDirName = (branchName == null) ? getWorkDirName() : branchName;
//        String tempWorkDirName = getWorkDirName();
        if (tempWorkDirName != null && !tempWorkDirName.equals("")) {
            tempWorkDirName += File.separator;
        }
        /*
        if (subModuleName != null) {
            localProjectDirName = tempWorkDirName + projectName + File.separator + 
                                  subModuleName;
        } else {
            // because we use the -N option for checkout we can append the 
            // module name to the work dir
            localProjectDirName = tempWorkDirName + projectName;
        }
        */
                
        localProjectDirName = tempWorkDirName + cvsModule;
        log.debug("localProjectDirName: "+localProjectDirName);

        // DO NOT TRY TO CREATE THE LOCAL PROJECT DIRECTORY, CVS WILL NOT WORK
    }

    /**
     * Returns a List of Revision objects detailing the changes that have 
     * been made since the specified date
     * 
     * @param Date 
     * @return List.
     */
    public List getRevisionsSince(Date date) 
    throws Exception {
        // you have to log in first
        if (!isLoggedIn()) {
            login();
        }
        
        String cvsroot = project.getProperties().getProperty(CVS_ROOT_KEY);
        String branch = getBranchName();

        // we must do a command array since using a command string in the 
        // Runitme.execute() method causes cvs to be unable to parse the dates
        String[] commandArray = null;
        List commandList = new ArrayList();
        commandList.add("cvs");
        commandList.add("-d");
        commandList.add(cvsroot);
        commandList.add("log");

        // if the date is null, then don't include it
        if (date != null) {

            // here we have to get the gmt version of this date. 
            // cvs works in gmt, and it doesn't always seem to behave
            // correctly if we send it est...
            TimeZone gmtTimeZone = TimeZone.getTimeZone("GMT");
	    CVS_IN_DATE.setTimeZone(gmtTimeZone);
            String dateConstraint = ">" +CVS_IN_DATE.format(date) + " GMT"; 

            commandList.add("-d" + dateConstraint + "");
        }
                            
        // if there is not branch, then don't include it
                
        if (branch != null) {
            commandList.add("-r" + branch);
        } else {
            // do not include revisions made on branches
            commandList.add("-rHEAD");
            commandList.add("-N"); // don't need tags if you don't have a branch (spb)
        }
        
        // don't print the tag info for the file
        // commandList.add("-N"); // commented out by (spb)

        // add the project directory whose log we want
        commandList.add(getLocalProjectDirName());

        // convert the list to the array
        commandArray = new String[commandList.size()];
        for (int i = 0; i < commandList.size(); i++) {
            commandArray[i] = (String)commandList.get(i);
        }
        
        // if we're debugging, then log the command
        String command = "";
        for (int i = 0; i < commandArray.length; i++) {
            command += commandArray[i] + " ";
        }
        log.debug("getRevisions: "+command);

        Process p = null;
        StreamPumper errorPumper = null;
        int exitcode = 0;
        List revisionList = new ArrayList();
        try {
            p = Runtime.getRuntime().exec(commandArray);

            // pump the error stream.
            errorPumper = new StreamPumper(p.getErrorStream(), "getRevisions", 
                                 System.err, log.isDebugEnabled());
            errorPumper.start();
            
            // get and parse the input stream
            InputStream input = p.getInputStream();
            parseLogCommandResult(input, revisionList);
            
            // test for errors
            exitcode = p.waitFor();

            if (exitcode == 0) {
                // verifyAddedAndRemovedFiles(revisionList); commented out by (spb)
                // this functionality is in the methods below
            }
            
        } finally {
            if (errorPumper != null) {
                errorPumper.join();
            }
        }

        // handle errors
        if (exitcode != 0) {
            throw (new Exception("cvs log failed"));
        }                
        
        // remove revisions attributable to the anthill user
        List cleanRevList = new ArrayList();
        String anthillUserName = project.getProperties().getProperty(CVS_ANTHILL_USER_KEY);
            log.debug("anthill user = " + anthillUserName);
        Revision rev = null;
        for (int i = 0; i < revisionList.size(); i++) {
            rev = (Revision) revisionList.get(i);
            if (!rev.userName.equals(anthillUserName)) {
                cleanRevList.add(rev);
            }
        }
        
        return cleanRevList;
    }

    protected void parseLogCommandResult(InputStream in, List revList) 
    throws IOException {
        BufferedReader reader = new BufferedReader(new InputStreamReader(in));
        List fileRevList = parseLogCommandResultFileBlock(reader);
        
        while (fileRevList != null) {
            revList.addAll(fileRevList);
            fileRevList = parseLogCommandResultFileBlock(reader);
        }
    }
    
    protected List parseLogCommandResultFileBlock(BufferedReader reader) 
    throws IOException {
        List revList = null;
        String line = null;
        
        // find start of file token
        line = reader.readLine();
        log.debug(line);
        while (line != null && !line.startsWith(WORKING_FILE_TOKEN)) {
            line = reader.readLine();
            log.debug(line);
        }

        // make sure there is a first file block
        if (line != null) {
            log.debug("***** Found WORKING_FILE_TOKEN ****");
            
            // get the working file name
            String fileName = line.substring(WORKING_FILE_TOKEN.length());
            
            // ADDED CODE (SPB)
            
            // if we are looking at a branch, there will be the symbolic list token
            // otherwise it won't find it, so don't even do this part
            List symList = new ArrayList();
            if (getBranchName() != null) { 
			
                while (line != null && !line.startsWith(SYMBOLIC_LIST_TOKEN)) {
                    line = reader.readLine();
                    log.debug(line);
                }
			
                if (line != null) {
                    // now we have hit the line that starts with the symbolic list token
                    // so we can start looking at the symbolic names.  
                    // They will start on the next line if there are any, if there aren't any
                    // you will see the keyword substitutions token
                    // They go by the <text tag name>: <revision>
                    line = reader.readLine();
                    while ((line != null) && (!line.startsWith(KEYWORD_SUBSTITUTIONS_TOKEN))) {
                        // here we are looking at a symbolic name. 
                        symList.add(line.substring(0, line.indexOf(":")).trim());
                        log.debug("*** Adding: " + line.substring(0, line.indexOf(":")).trim() + " ***");
                        line = reader.readLine();
                    }
                    
                    log.debug("Found keyword substitutions");
                }
            }
            // END ADDED CODE (SPB)
            
            // build the revision list
            revList = parseLogCommandResultRevisionBlock(reader, fileName, symList);
        }
        
        return revList;
    }
    
    protected List parseLogCommandResultRevisionBlock(BufferedReader reader, 
                                                          String fileName, List symList) 
    throws IOException {
        List revList = new ArrayList();
        String line = null;
        boolean newFile = false;
        int totalRevs = 0;
        boolean wasAdded = false;

        // find start of revision token
        line = reader.readLine();
        log.debug(line);
        // make sure that we did not start a new file
        if (line.startsWith(FILE_DELIM_TOKEN)) {
            newFile = true;
        }
        
        while (!newFile) {
            // look for the start of a revision
            while (line != null && !newFile 
                                && !line.startsWith(REV_NUMB_TOKEN)) {
                line = reader.readLine();
                log.debug(line);

                // check that we did not start a new file
                if (line.startsWith(FILE_DELIM_TOKEN)) {
                    newFile = true;
                } 
                
                // commented out by (spb), shouldn't need this
           /*     else if (line.startsWith(REV_REVISIONS_TOKEN)) {
                    try {
                        totalRevs = Integer.parseInt(
                            line.substring(REV_REVISIONS_TOKEN.length(), 
                                           line.indexOf(';')).trim());
                        wasAdded = (totalRevs == 1);
                    } catch (Exception e) {
                        log.debug(e);
                    }
                }
            */
                
            }

            // make sure that we found the beginning of the revision
            if (line != null && !newFile) {
                // now parse the revision block
                log.debug("***** Found REV_NUMB_TOKEN ****");
                String revNum = line.substring(REV_NUMB_TOKEN.length(), line.length());
                // find the line that starts with the REV_DATE_TOKEN
                line = reader.readLine();
                log.debug(line);
                while (line != null && !newFile 
                                    && !line.startsWith(REV_DATE_TOKEN)) {
                    line = reader.readLine();
                    log.debug(line);

                    // check that we did not start a new file
                    if (line.startsWith(FILE_DELIM_TOKEN)) {
                        newFile = true;
                    }
                }

                // make sure there is another line
                if (line != null && !newFile) {
                    log.debug("***** Found REV_DATE_TOKEN ****");
                    Revision rev = new Revision();
                    //if (revList == null) revList = new ArrayList();
                    
                    // commented next line out, don't want to add it yet
                    // not sure if it's an actual revision we need to worry about
           //         revList.add(rev);
                    rev.fileName = fileName;

                    // the date, author, and status are all on the same line
                    StringTokenizer tokenizer = new StringTokenizer(line, ";,  ");

                    // first token is the REV_DATE_TOKEN followed by the date then time
                    tokenizer.nextToken();
                    String date = tokenizer.nextToken();
                    String time = tokenizer.nextToken();
                    try {
                        rev.date = CVS_OUT_DATE.parse(date + " " + time);
                    } catch (Exception e) {
                        log.error("Can't parse revision date.", e);
                    }

                    // next token is the author keyword followed by the user name
                    tokenizer.nextToken();
                    rev.userName = tokenizer.nextToken();

                    // next token is the state keyword followed by the state of the rev
                    // if the state is dead, this file could either have been deleted
                    // from the current directory, or it was added on a different branch
                    // above we recorded the revision number, if it was 1.1, and the state
                    // is dead, this file was added on another branch, so you can ignore
                    // this revision, if the revision wasn't 1.1, this file was deleted 
                    // from the current directory, which we do need to take notice of
                    tokenizer.nextToken();
                    String stateToken = tokenizer.nextToken();
                    if (stateToken.equals(REV_STATE_DEAD_TOKEN)) {
                        if (revNum.trim().equals("1.1")) {
                            rev.wasDeleted = false;
                            rev.wasAdded = true;
                        }
                        else {
                            rev.wasDeleted = true;
                            rev.wasAdded = false;
                        }
                    }
                    
                    // it's now time to decide if we want to keep the current revision
                    // we are looking at.  This depends on whether we have a branch or
                    // not.  if we do, need to consult the symbolic list, if the branch
                    // isn't in the list, the revision can be ignored.  
                    // if we are looking at the head, really only have to look at the
                    // rev.wasAdded variable, if it was added, which really means was
                    // added on another branch, then we can ignore it.
                    
                    String branchName = getBranchName();
                    if (branchName == null) {
                        // here we put in -rHEAD
                        if (!rev.wasAdded) {
                            // revision should be valid
                            revList.add(rev);
                        }
                    }
                    else {
                        if (symList.contains(branchName.trim())) {
                            // here check if it was added again
                            log.debug("SymList contained branch name: " + branchName);
                            if (!rev.wasAdded) {
                                log.debug("Wasn't added!  Need to build!");
                                revList.add(rev);
                            }
                        }
                        else {
                            log.debug("SymList did NOT contain branch name: " + branchName);
                            log.debug("list contents:");
                            for (int i = 0; i < symList.size(); i++) {
                                log.debug((String)symList.get(i));
                            }
                        }
                    }
                                     
                    // next token is the lines keyword followed by the lines added and 
                    // then lines deleted.  These tokens are not always there
                    
                    /* commented out by (spb) shouldn't need this stuff either
                    if (tokenizer.hasMoreTokens()) {
                        tokenizer.nextToken();
                        String linesAdded = tokenizer.nextToken();
                        linesAdded = linesAdded.substring(1);
                        String linesDeleted = tokenizer.nextToken();
                        linesDeleted = linesDeleted.substring(1);
                        rev.linesAdded = Integer.parseInt(linesAdded);
                        rev.linesDeleted = Integer.parseInt(linesDeleted);
//                        rev.wasAdded = false;
//                    } else {
                        // if no lines added/deleted appears, it means the file
                        // is new
//                        rev.wasAdded = true;
                    }
                     
                     */
                
                    //    rev.wasAdded = wasAdded;  commented out by (spb)
                    log.debug(fileName + " has rev.wasAdded=" + rev.wasAdded);
                    
                    // the comment may take up more than one line.  We don't know how 
                    // lines the comment does take up but we know that once we reach 
                    // the start of the next revision or the next file then the comment 
                    // has ended.
                    rev.comment = new String();
                    line = reader.readLine();
                    log.debug(line);

                    // make sure we did not start a new file
                    if (line.equals(FILE_DELIM_TOKEN)) {
                        newFile = true;
                    }
                        
                    // read lines until we start a new revision or a new file
                    while (line != null && !newFile 
                                        && !line.equals(REV_DELIM_TOKEN) 
                                        && !line.equals(FILE_DELIM_TOKEN)) {
                        rev.comment += line + NEW_LINE;
                        line = reader.readLine();
                        log.debug(line);

                        // if we started a new file, set the flag
                        if (line.equals(FILE_DELIM_TOKEN)) {
                            newFile = true;
                        }
                    }
                }
            }
        }
        return revList;
    }

    protected void verifyAddedAndRemovedFiles(List revisionList)
    throws Exception {
        String[] commandArray = null;
        List commandList = new ArrayList();
        Revision revision = null;
        File revisedFile = null;

        // the revised list of revisions
        List tempRevisionList = new ArrayList();
        
        Iterator revItr = revisionList.iterator();
        while (revItr.hasNext()) {
            revision = (Revision)revItr.next();
            
            if (revision.wasAdded) {
                // make sure file exists locally, otherwise don't include this
                // revision
                log.debug("checking added file: " + revision.fileName);
                revisedFile = new File(revision.fileName);
                if (revisedFile.exists()) {
                    tempRevisionList.add(revision);
                }
            } else if (revision.wasDeleted) {
                // make sure file doesn't exist locally, otherwise don't include
                // this revision
                log.debug("checking removed file: " + revision.fileName);
                revisedFile = new File(revision.fileName);
                if (!revisedFile.exists()) {
                    tempRevisionList.add(revision);
                }
            } else {
                tempRevisionList.add(revision);
            }
        }
        
        revisionList.clear();
        revisionList.addAll(tempRevisionList);
    }
    
}
