/*
 * @(#)RepositoryAdapter.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.adapter;

import org.apache.log4j.Category;
import java.io.File;
import com.urbancode.anthill.AnthillProject;
import java.util.List;
import java.util.Date;
/**
 * <p>The abstract class representing a type or means of communication with 
 * a repository.</p>
 *
 * @author  Maciej Zawadzki
 * @author  Ryan Smith
 */
public abstract class RepositoryAdapter {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(RepositoryAdapter.class.getName());
    
    // work dir property key
    static public final String WORK_DIR_KEY = "repository.cvs.work.dir";
    // key with property informing us whether the work dir has to be relative
//    static public final String WORK_DIR_RELATIVE_KEY = "work.dir.relative";

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    private boolean isLoggedIn = false;
    protected String workDirName = null;
    protected AnthillProject project = null;
    protected String localProjectDirName = null;
    protected String branchName = null;
    
    /**
     * Returns the AnthillProject that this RepositoryAdapter belongs to.
     *
     * @return AnthillProject that this RepositoryAdapter belongs to
     */
    public AnthillProject getAnthillProject() {
        return project;
    }
    
    /** 
     * Check to see if the adapter is logged into the repository.
     *
     * @return  true if logged in, false if not
     */
    public boolean isLoggedIn() {
        return(isLoggedIn);
    }
    
    /**
     * Sets that the adapter is logged into the repository.
     */
    protected void setLoggedIn() {
        isLoggedIn = true;
    }
    
    /**
     * Sets that the adapter is NOT logged into the repository.
     */
    protected void setLoggedOut() {
        isLoggedIn = false;
    }
    
    /**
     * Log into the repository.
     */
    public abstract void login() 
    throws Exception;
    
    /**
     * Log out of the repository.
     */
    public abstract void logout() 
    throws Exception;
    
    /**
     * Checkout the project from the repository.
     */
    public abstract void checkout() 
    throws Exception;
    
    /**
     * Lock a file with the repository.
     *
     * @param file  the name of the file to lock
     */
    public abstract void lockFile(String file) 
    throws Exception;
    
    /**
     * Unlock a file with the repository.
     *
     * @param file  the name of the file to unlock
     */
    public abstract void unlockFile(String file) 
    throws Exception;

    /**
     * Update a file from the repository.
     *
     * @param file  the name of the file to update
     */
    public abstract void updateFile(String file) 
    throws Exception;
    
    /**
     * Commit a file to the repository.
     *
     * @param file  the name of the file to commit
     */
    public abstract void commitFile(String file, String msg) 
    throws Exception;
    
    /**
     * Tag a project in the repository.
     *
     * @param tag  the value of the tag to tag the project with
     */
    public abstract void tagProject(String tag) 
    throws Exception;
    
    public abstract List getRevisionsSince(Date date)
    throws Exception;
    
    /**
     * Get the work directory for the repository adapter.
     * The work directory is the location to which the adapter can checkout 
     * files from the repository.
     *
     * @return  the name of the working directory for the repository adapter
     */
    public String getWorkDirName() {
        return(workDirName);
    }
    
    /**
     * Returns the name of the local directory that holds the checked out 
     * sources of the project.
     *
     * @return name of the local project directory
     */
    public String getLocalProjectDirName() {
        return localProjectDirName;
    }
    
    /**
     * Set the AnthillProject that this RepositoryAdapter belongs to.
     * This method should only be called from the RepositoryAdapterFactory.
     *
     * @param the AnthillProject that this adapter belongs to
     */
    protected void setAnthillProject(AnthillProject project) 
    throws Exception {
        this.project = project;
        calculateBranchName();
        calculateWorkDirName();
    }
    
    /**
     * Calculates the work directory for this repository adapter.
     * The work directory is the location to which the adapter can checkout 
     * files from the repository.
     */
    protected void calculateWorkDirName() 
    throws Exception {
        // cvs does not like absolute directories under unix
        /*
        File rootDir = project.getRootDir();
        workDirName = rootDir.getAbsolutePath() + File.separator;
        workDirName = workDirName + project.getProperty(WORK_DIR_KEY);
         */
        workDirName = project.getProperties().getProperty(WORK_DIR_KEY);
        
        // make sure that the work dir is not null
        if (workDirName == null) {
            throw new IllegalStateException("work dir property not set");
        } else {
            /*
             * CVS on unix won't take a multi-level directory for the
             * "checkout -d" option.
             */
/*            
            String branchName = getBranchName();
            if (branchName != null) {
                if (workDirName.endsWith(File.separator)) {
                    workDirName += branchName;
                } else {
                    workDirName += File.separator + branchName;
                }
            }
*/                
            // check that the working directory does not end with "/"
            if (workDirName.endsWith(File.separator)) {
                workDirName = workDirName.substring(0, workDirName.length()-1);
            }
        }
        // DO NOT TRY TO CREATE THE WORK DIRECTORY, CVS WILL NOT WORK
        
        calculateLocalProjectDirName();
    }
    
    /**
     * This method calculates the name of the local directory where the 
     * checkedout project sources will be (are) placed.  The default 
     * implementation returns workDirName/projectName.
     */
    protected void calculateLocalProjectDirName() {
        // make sure that the work dir name has been set
        if (workDirName == null) {
            throw new IllegalStateException(
                "CVSRepositoryAdapter.calculateLocalProjectDirName() called " +
                "before the work directory name was set.");
        }
        
        // set the localProjectDirName 
        String projectName = getAnthillProject().getProjectName();
        log.debug("projectName: "+projectName);

        String tempWorkDirName = getWorkDirName();
        if (tempWorkDirName != null && !tempWorkDirName.equals("")) {
            tempWorkDirName += File.separator;
        }
        localProjectDirName = tempWorkDirName + projectName;
        log.debug("localProjectDirName: "+localProjectDirName);

        // DO NOT TRY TO CREATE THE LOCAL PROJECT DIRECTORY, CVS WILL NOT WORK
    }
        
    /**
     * Returns the name of the branch that the project is on.
     *
     * @return name of the branch that the project is on
     */
    public String getBranchName() {
        return branchName;
    }
    
    /**
     * Calculates the branch that this project is on, if any.
     */
    abstract protected void calculateBranchName();
    
}