/*
 * @(#)UrbanCodeVersionAdapter.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.adapter;

import org.apache.log4j.Category;
import com.urbancode.lib.version.VersionModifier;
import com.urbancode.anthill.*;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.File;
/**
 * <p>
 * A implementation of <code>VersionAdapter</code> that does versioning in
 * a single version file that is of the registry file type (see Butler project)
 * with the extension .version.</p>
 * <p>
 * When the version is incremented, the version file is also immediately
 * modified.</p>
 * <p>
 * The options for increment are:
 * <ul>
 *   <li>major - would increment 1.1.1 to 2.0.0
 *   <li>minor - would increment 1.1.1 to 1.2.0
 *   <li>build - would increment 1.1.1 to 1.1.2
 * </ul></p>
 *
 * @author  Ryan Smith
 */
public class UrbanCodeVersionAdapter extends VersionAdapter {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(UrbanCodeVersionAdapter.class.getName());
    
    // version file property key
    static public final String VERSION_FILE_KEY = "version.file";
    // lock version file flag property key
    static public final String LOCK_VERSION_FLAG_KEY = "version.lock.file";
    
    static private final int PREFIX = 1;
    static private final int DIGIT = 2;
    static private final int POSTFIX = 3;

    //**************************************************************************
    // INSTANCE
    //**************************************************************************
    protected String versionFile = null;
    protected ProjectProperties properties = null;
    protected RepositoryAdapter repositoryAdapter = null;

    /**
     * Create a new <code>UrbanCodeVersionAdapter</code>.
     */
    public UrbanCodeVersionAdapter() {
    }
    
    protected void setAnthillProject(AnthillProject project) {
        this.project = project;
        this.properties = project.getProperties();
        this.repositoryAdapter = properties.getRepositoryAdapter();

        calculateVersionFile();
    }
    
    /**
     * Calculates the project version file to be incremented.
     */
    protected void calculateVersionFile() {
        String localVersionFilePath =  
            project.getRepositoryAdapter().getLocalProjectDirName() + 
            File.separator + properties.getProperty(VERSION_FILE_KEY);
        this.versionFile = localVersionFilePath;
    }
    
    /**
     * Increment the project with the version increment. Immediately updates
     * the version file with the new version and returns the new version.
     *
     * @return  string representation of the new version of the project
     */
    public String incrementVersion() 
    throws Exception {
        String branch = project.getRepositoryAdapter().getBranchName();
        
        // make sure the version file was set
        if (versionFile == null)  
            throw new IllegalStateException("no version file set.");
        
        // get the Version from the version file
        File f = new File(versionFile);
        
        log.info("Incrementing version file: "+f.getPath());
        
        VersionModifier modifier = new VersionModifier(f);
        String versionStr = modifier.readVersionStr();
        String newVersionStr = null;
        
        if (branch != null) {
            if (versionStr.indexOf(branch) == -1) {
                throw new IllegalStateException("Version string is illegal.  " +
                            "It does not contain the Branch name. " +
                            "versionStr: " + versionStr);
            }
        }
        
        // lock the version file is required
        doPreVersionFileModify();
        
        newVersionStr = incrementVersion0(versionStr);
        modifier.writeVersionStr(newVersionStr);
        
        // commit and unlock the file
        doPostVersionFileModify();

        f = null;
        modifier = null;

        //  return the new version property
        return (newVersionStr);
    }
    
    public String getCurrentVersion()
    throws Exception {        
        // make sure the version file was set
        if (versionFile == null)  
            throw new IllegalStateException("no version file set.");
        
        // get the Version from the version file
        File f = new File(versionFile);
        
        log.info("getting version file: "+f.getPath());
        
        VersionModifier modifier = new VersionModifier(f);
        String versionStr = modifier.readVersionStr();
        return versionStr;
    }

    public void setCurrentVersion(String newVersion)
    throws Exception {        
        // make sure the version file was set
        if (versionFile == null)  
            throw new IllegalStateException("no version file set.");
        
        // get the Version from the version file
        File f = new File(versionFile);
        
        log.info("setting version file: "+f.getPath());
        
        // lock the version file is required
        doPreVersionFileModify();
        
        VersionModifier modifier = new VersionModifier(f);
        modifier.writeVersionStr(newVersion);        
        
        // commit and unlock the file
        doPostVersionFileModify();
    }
    
    protected String incrementVersion0(String versionStr) {
        // incrementing only the build number
        char c;
        StringBuffer prefix = new StringBuffer();
        StringBuffer digits = new StringBuffer();
        StringBuffer postfix = new StringBuffer();
        int state = POSTFIX;

        for (int i = versionStr.length() - 1; i >= 0; i--) {
            c = versionStr.charAt(i);
            if (!Character.isDigit(c)) {
                if (state == POSTFIX) {
                    postfix.insert(0, c);
                } else {
                    if (state == DIGIT) {
                        state = PREFIX;
                    }
                    prefix.insert(0, c);
                }
            } else {
                if (state == POSTFIX || state == DIGIT) {
                    state = DIGIT;
                    digits.insert(0, c);
                } else {
                    prefix.insert(0, c);
                }
            }
        }

        int build = 0;
        try {
            build = Integer.parseInt(digits.toString());
        } catch (NumberFormatException nfe) {
            throw new IllegalStateException("Version String illegal. " + 
                        "Can't parse build number. " +
                        "versionStr: "+versionStr);
        }
        build++;

        String newVersionStr = prefix.toString() + String.valueOf(build) + 
                               postfix.toString();
        
        return newVersionStr;
    }

    protected void doPreVersionFileModify() 
    throws Exception {
        // lock the version file in the repository
        // check lock version property first
        String lockVersionFlagStr = properties.getProperty(LOCK_VERSION_FLAG_KEY);
        if (lockVersionFlagStr != null && lockVersionFlagStr.equalsIgnoreCase("off")) {
            log.info("version file locking turned off");
        } else {
            log.info("Lock version file: ");
            repositoryAdapter.lockFile(versionFile);
//                            releaseState.setVersionFileLockedFlag(true);
        }
    }
    
    protected void doPostVersionFileModify() 
    throws Exception {
        // update the version file in the repository
        log.info("Update version file: ");
        repositoryAdapter.updateFile(versionFile);

        // commit the version file to the repository
        log.info("Commit version file: ");
        repositoryAdapter.commitFile(versionFile, null);
    }
    
}