/*
 * @(#)FileUtils.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.util;
import org.apache.log4j.Category;
import java.io.*;
import java.util.Locale;
import java.text.NumberFormat;
/**
 * @version $Revision: 1.3 $
 * @author 
 */
public abstract class FileUtils {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    // Create Log4j category instance for logging
    static private Category log = Category.getInstance(FileUtils.class.getName());
    
    //--------------------------------------------------------------------------
    /**
     * This method checks whether the path passed in is a relative path.  
     * If so, then it uses the root parameter as a base for the relative path 
     * and returns the resulting file.  If the path is not relative, then 
     * a file corresponding to the path is returned.
     */
    static public File makeProperPath(File root, String path) {
        if (root == null) {
            throw new IllegalArgumentException("Root parameter must not be " +
                                               "null.");
        }
        if (path == null) {
            throw new IllegalArgumentException("Path parameter must not be " +
                                               "null.");
        }
        
        File properPath = null;
        
        // it is not a relative path if it starts with "/" or contains ":\"
        boolean relative = true;
        if (path.startsWith("/")) {
            relative = false;
        }
        if (path.indexOf(":\\") > 0) {
            relative = false;
        }
        
        if (relative) {
            properPath = new File(root, path);
        } else {
            properPath = new File(path);
        }
//        log.debug("Root: "+root.toString() + ", path: "+path);
//        log.debug("ProperPath: "+properPath.toString());
        return properPath;
    }
    
    //--------------------------------------------------------------------------
    static public String getNameMinusExtension(File file) {
        String name = file.getName();
        int index = name.lastIndexOf(".");
        name = name.substring(0, index);
        return name;
    }
    
    // Data Sizes
    static public final double KB = 1024;
    static public final double MB = KB * KB;
    static public final double GB = KB * MB;
    
    //--------------------------------------------------------------------------
    static public String getBytes(File file) {
        return getBytes(file, Locale.getDefault());
    }
    static public String getBytes(File file, Locale locale) {
        NumberFormat format = NumberFormat.getInstance(locale);
        format.setMaximumFractionDigits(2);
        return format.format(file.length()) + " bytes";
    }
    
    //--------------------------------------------------------------------------
    static public String getKiloBytes(File file) {
        return getKiloBytes(file, Locale.getDefault());
    }
    static public String getKiloBytes(File file, Locale locale) {
        NumberFormat format = NumberFormat.getInstance(locale);
        format.setMaximumFractionDigits(2);
        double size = (double) file.length(); 
        return format.format(size / KB) + " KB";
    }
    
    //--------------------------------------------------------------------------
    static public String getMegaBytes(File file) {
        return getMegaBytes(file, Locale.getDefault());
    }
    static public String getMegaBytes(File file, Locale locale) {
        NumberFormat format = NumberFormat.getInstance(locale);
        format.setMaximumFractionDigits(2);
        double size = (double) file.length(); 
        return format.format(size / MB) + " MB";
    }
    
    //--------------------------------------------------------------------------
    static public String getGigaBytes(File file) {
        return getGigaBytes(file, Locale.getDefault());
    }
    static public String getGigaBytes(File file, Locale locale) {
        NumberFormat format = NumberFormat.getInstance(locale);
        format.setMaximumFractionDigits(2);
        double size = (double) file.length(); 
        return format.format(size / GB) + " GB";
    }
    
    //--------------------------------------------------------------------------
    static public String getNearestBytes(File file) {
        return getNearestBytes(file, Locale.getDefault());
    }
    static public String getNearestBytes(File file, Locale locale) {
        long size = file.length();
        if (size >= GB) {
            return getGigaBytes(file, locale);
        } else if (size >= MB) {
            return getMegaBytes(file, locale);
        } else if (size >= KB) {
            return getKiloBytes(file, locale);
        } else {
            return getBytes(file, locale);
        }
    }
    
    //**************************************************************************
    // INSTANCE
    //**************************************************************************

    //--------------------------------------------------------------------------

}
