/*
 * @(#)StreamPumper.java
 *
 * License -
 * The contents of this file are subject to the Urbancode Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.urbancode.com/licenses/UPL/1_0/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Original Code is Urbancode Software
 * Development, Inc. ("Urbancode").  Portions created by Urbancode are
 * Copyright (C) Urbancode Software Development, Inc. All Rights Reserved.
 */
package com.urbancode.anthill.util;

import java.io.*;
/**
 * <p>
 * This class attaches to a <code>InputStream</code> and reads the input. It
 * will also log the input to an <code>OutputStream</code> that is specified if
 * set to verbose mode. The default mode is NOT verbose and writes nothing to
 * the OutputStream.</p>
 *
 * @author Ryan Smith
 * @author Maciej Zawadzki
 * @version $Revision 1.4 $
 */
public class StreamPumper extends Thread {

    private final static int SIZE = 128;
    private final static int SLEEP_TIME = 50;
    
    private InputStream is = null;
    private OutputStream os = null;
    private String name = null;
    private boolean verbose = false;

    /**
     * Create a new <code>StreamPumper</code>. Verbose is false by default, so
     * nothing will be written to the writer.
     *
     * @param is  the <code>InputStream</code> to read
     * @param name  the arbitrary name to use when writing to the writer
     * @param writer  the <code>PrintWriter</code> to write the input out to
     */
    public StreamPumper(InputStream is, String name, OutputStream os) {
        this(is, name, os, false);
    }

    /**
     * Create a new <code>StreamPumper</code>.
     *
     * @param is  the <code>InputStream</code> to read
     * @param name  the arbitrary name to use when writing to the writer
     * @param writer  the <code>PrintWriter</code> to write the input out to
     * @param verbose  writes the input read to the writer if true
     */
    public StreamPumper(InputStream is, String name, OutputStream os, boolean verbose) {
        this.is = is;
        this.os = os;

        if (name != null) {
            this.name = "["+name+"] ";
        } else {
            this.name = "";
        }
        this.verbose = verbose;
    }
    
    /**
     * Start listening to the <code>InputStream</code>.
     */
    public void run() {
        byte[] buf = new byte[SIZE];
        int length = 0;
        String temp = "";
        
        try {
            while ((length = is.read(buf)) > 0) {
                if (verbose) {
                    temp += new String(buf, 0, length);
                    int index = temp.lastIndexOf("\n");
                    if (index != -1) {
                        String line = temp.substring(0, index+1);
                        temp = temp.substring(index+1);
                        if (os != null) os.write(line.getBytes());
                    }
                }
                try {
                    sleep(SLEEP_TIME);
                } catch (InterruptedException e) {}
            }
            if (os != null) os.write(temp.getBytes());
        } catch(IOException e) {}
    }
}
